﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Lokeiv.Games.Ludo.Model;
using System.Xml.Serialization;
using System.Xml;

namespace Lokeiv.Games.Ludo
{
    public class DeepSerializer
    {
        public Engine InitializeEngineToBeSerialized()
        {
            PlayerCollection players = new PlayerCollection();            
            players.Add(new Player("Red player", PlayerType.Human, Lokeiv.Games.Ludo.Model.Color.Red));
            players.Add(new Player("Yellow player", PlayerType.Human, Lokeiv.Games.Ludo.Model.Color.Yellow));
            players.Add(new Player("Green player", PlayerType.Computer, Lokeiv.Games.Ludo.Model.Color.Green));
            players.Add(new Player("Blue player", PlayerType.Computer, Lokeiv.Games.Ludo.Model.Color.Blue));
            GameOptions gameOptions = new GameOptions();
            gameOptions.GameType = GameType.Classic;
            gameOptions.FormBlocks = true;
            gameOptions.RollThreeTimes = true;

            Lokeiv.Games.Ludo.Engine engine = new Lokeiv.Games.Ludo.Engine(players, gameOptions);

            return engine;
        }

        public string SerializeToXml(Engine engine)
        {
            XmlSerializer serializer = new XmlSerializer(typeof(Engine));

            using (StringWriter stringWriter = new StringWriter())
            {
                serializer.Serialize(stringWriter, engine);
                string data = stringWriter.ToString();

                return data;
            }
        }

        public Engine DeserializeEngine(string xml)
        {
            var engine = new Engine();
            XmlSerializer serializer = new XmlSerializer(typeof(Engine));

            using (StringReader stringReader = new StringReader(xml))
            using (XmlReader xmlReader = XmlReader.Create(stringReader))
            {
                if (serializer.CanDeserialize(xmlReader))
                    try
                    {
                        engine = (Engine)serializer.Deserialize(xmlReader);
                    }
                    catch (InvalidOperationException ioex)
                    {
                        throw new Exception("Unable to load game state. See inner exception for details.", ioex);
                    }

            }

            return engine;
        }

        public Engine RunEngineToBeSerialized()
        {
            Engine engine = this.InitializeEngineToBeSerialized();
            engine.StartGame();

            // Todo: Subscribe to events and react to play the game for some time
            engine.EndGame();

            return engine;
        }
    }    
}
