using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;
using Ascend;
using Ascend.Windows.Forms;

namespace Lokeiv.Games.Ludo.Forms.Client
{
    public class GoalRender : GradientBackgroundRender
    {
        #region Enumerations
        public enum PlacementEnum
        {
            North, East, South, West
        }
        #endregion
        
        private RectangleF _goalBounds;
        private Color _gradientWhiteHighColor;
        private PlacementEnum _placement;
        private Border _border;
        private Color _adjustedBorderColorTop;	

        public new Border Border
        {
            get { return _border; }
            set { _border = value; }
        }

        public PlacementEnum Placement
        {
            get { return _placement; }
            set { _placement = value; }
        }

        public Color GradientWhiteHighColor
        {
            get { return _gradientWhiteHighColor; }
            set { _gradientWhiteHighColor = value; }
        }

        public RectangleF GoalBounds
        {
            get { return _goalBounds; }
            set { _goalBounds = value; this.DisplayRectangle = new Rectangle((int)value.X, (int)value.Y, (int)value.Width, (int)value.Height); }
        }

        public GoalRender(Lokeiv.Games.Ludo.Model.Color color)
        {
            this.Padding = new System.Windows.Forms.Padding(0);
            this.Border = new Border(1);
            this.BorderColor = new BorderColor(System.Drawing.Color.Black);

            // Select the color to draw this house with
            switch (color)
            {
                case Lokeiv.Games.Ludo.Model.Color.Red:
                    this.GradientHighColor = System.Drawing.Color.LightCoral;
                    this.GradientLowColor = System.Drawing.Color.DarkRed;
                    this._placement = PlacementEnum.West;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.Yellow:
                    this.GradientHighColor = System.Drawing.Color.LightYellow;
                    this.GradientLowColor = System.Drawing.Color.Yellow;
                    this.Placement = PlacementEnum.North;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.Green:
                    this.GradientHighColor = System.Drawing.Color.LightGreen;
                    this.GradientLowColor = System.Drawing.Color.DarkGreen;
                    this.Placement = PlacementEnum.East;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.Blue:
                    this.GradientHighColor = System.Drawing.Color.LightBlue;
                    this.GradientLowColor = System.Drawing.Color.DarkBlue;
                    this.Placement = PlacementEnum.South;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.White:
                    this.GradientHighColor = System.Drawing.Color.White;
                    this.GradientLowColor = System.Drawing.Color.WhiteSmoke;
                    break;
            }  
        }

        public override void Render(System.Windows.Forms.PaintEventArgs e)
        {
            if (e == null)
            {
                throw new ArgumentNullException("e");
            }

            RectangleF bounds;
            //float topOffset = (this._posOffset + base.Padding.Top);
            //float leftPosition = (this.DisplayRectangle.Left + base.Padding.Left + this._posOffset);
            //float topPosition = (this.DisplayRectangle.Top + this._posOffset + base.Padding.Top);
            //float rightOffset = (this._posOffset + base.Padding.Right);
            //float bottomOffset = (this._posOffset + base.Padding.Bottom);
            //bounds = new RectangleF(leftPosition, topPosition, (this.DisplayRectangle.Width - (rightOffset)), (this.DisplayRectangle.Height - (topOffset + bottomOffset)));
            bounds = new RectangleF(this.DisplayRectangle.Left, this.DisplayRectangle.Top, this.DisplayRectangle.Width, this.DisplayRectangle.Height);
            PointF triangleTopPoint = new PointF(0, 0);
            
            switch(this.Placement)
            {
                case PlacementEnum.West:
                    triangleTopPoint = new PointF(bounds.X + bounds.Width, bounds.Y + bounds.Height / 2.0f);
                    break;
                case PlacementEnum.North:
                    triangleTopPoint = new PointF(bounds.X + bounds.Width / 2.0f, bounds.Y + bounds.Height);
                    break;
                case PlacementEnum.East:
                    triangleTopPoint = new PointF(bounds.X, bounds.Y + bounds.Height / 2.0f);
                    break;
                case PlacementEnum.South:
                    triangleTopPoint = new PointF(bounds.X + bounds.Width / 2.0f, bounds.Y);
                    break;
            }
            
            if (this.DisplayRectangle.Height > 0 && this.DisplayRectangle.Width > 0)
            {
                this.CreateBackBrush();

                GraphicsPath backgroundPath = new GraphicsPath();

                switch (this.Placement)
                {
                    case PlacementEnum.West:
                        backgroundPath.AddLine(bounds.X, bounds.Y, triangleTopPoint.X, triangleTopPoint.Y);
                        backgroundPath.AddLine(triangleTopPoint.X, triangleTopPoint.Y, bounds.X, bounds.Y + bounds.Height);
                        backgroundPath.AddLine(bounds.X, bounds.Y + bounds.Height, bounds.X, bounds.Y);
                        break;
                    case PlacementEnum.North:
                        backgroundPath.AddLine(bounds.X + bounds.Width, bounds.Y, triangleTopPoint.X, triangleTopPoint.Y);
                        backgroundPath.AddLine(triangleTopPoint.X, triangleTopPoint.Y, bounds.X, bounds.Y);
                        backgroundPath.AddLine(bounds.X, bounds.Y, bounds.X + bounds.Width, bounds.Y);
                        break;
                    case PlacementEnum.East:
                        backgroundPath.AddLine(bounds.X + bounds.Width, bounds.Y + bounds.Height, triangleTopPoint.X, triangleTopPoint.Y);
                        backgroundPath.AddLine(triangleTopPoint.X, triangleTopPoint.Y, bounds.X + bounds.Width, bounds.Y);
                        backgroundPath.AddLine(bounds.X + bounds.Width, bounds.Y, bounds.X + bounds.Width, bounds.Y + bounds.Height);
                        break;
                    case PlacementEnum.South:
                        backgroundPath.AddLine(bounds.X, bounds.Y + bounds.Height, triangleTopPoint.X, triangleTopPoint.Y);
                        backgroundPath.AddLine(triangleTopPoint.X, triangleTopPoint.Y, bounds.X + bounds.Width, bounds.Y + bounds.Height);
                        backgroundPath.AddLine(bounds.X + bounds.Width, bounds.Y + bounds.Height, bounds.X, bounds.Y + bounds.Height);
                        break;
                }

                e.Graphics.FillPath(this.BackBrush, backgroundPath);
            }

            // Draw border
            if (this._border.Top > 0)
            {
                this._adjustedBorderColorTop = this.BorderColor.Top;
                Pen borderPen = new Pen(this._adjustedBorderColorTop, this._border.Top);
                try
                {
                    float offset = 0; // (float)(this._border.Top / 2.0);
                    PointF triangleBorderTopPoint = new PointF(0, 0);

                    switch (this.Placement)
                    {
                        case PlacementEnum.West:
                            triangleBorderTopPoint = new PointF(this.DisplayRectangle.Left + this.DisplayRectangle.Width, this.DisplayRectangle.Y + this.DisplayRectangle.Height / 2.0f);
                            e.Graphics.DrawLine(borderPen, this.DisplayRectangle.Left, this.DisplayRectangle.Top + offset, triangleBorderTopPoint.X + offset, triangleBorderTopPoint.Y + offset);
                            e.Graphics.DrawLine(borderPen, triangleBorderTopPoint.X + offset, triangleBorderTopPoint.Y + offset, this.DisplayRectangle.Left, this.DisplayRectangle.Top + this.DisplayRectangle.Height);
                            e.Graphics.DrawLine(borderPen, this.DisplayRectangle.Left, this.DisplayRectangle.Top + this.DisplayRectangle.Height, this.DisplayRectangle.Left, this.DisplayRectangle.Y);
                            break;
                        case PlacementEnum.North:
                            triangleBorderTopPoint = new PointF(this.DisplayRectangle.Left + this.DisplayRectangle.Width / 2.0f, this.DisplayRectangle.Top + this.DisplayRectangle.Height);
                            e.Graphics.DrawLine(borderPen, this.DisplayRectangle.Left + this.DisplayRectangle.Width, this.DisplayRectangle.Top, triangleBorderTopPoint.X, triangleBorderTopPoint.Y);
                            e.Graphics.DrawLine(borderPen, triangleBorderTopPoint.X, triangleBorderTopPoint.Y, this.DisplayRectangle.Left, this.DisplayRectangle.Top);
                            e.Graphics.DrawLine(borderPen, this.DisplayRectangle.Left, this.DisplayRectangle.Top, this.DisplayRectangle.Left + this.DisplayRectangle.Width, this.DisplayRectangle.Top);
                            break;
                        case PlacementEnum.East:
                            triangleBorderTopPoint = new PointF(this.DisplayRectangle.Left, this.DisplayRectangle.Top + this.DisplayRectangle.Height / 2.0f);
                            e.Graphics.DrawLine(borderPen, this.DisplayRectangle.Left + this.DisplayRectangle.Width, this.DisplayRectangle.Top + this.DisplayRectangle.Height, triangleBorderTopPoint.X, triangleBorderTopPoint.Y);
                            e.Graphics.DrawLine(borderPen, triangleBorderTopPoint.X, triangleBorderTopPoint.Y, this.DisplayRectangle.Left + this.DisplayRectangle.Width, this.DisplayRectangle.Top);
                            e.Graphics.DrawLine(borderPen, this.DisplayRectangle.Left + this.DisplayRectangle.Width, this.DisplayRectangle.Top, this.DisplayRectangle.Left + this.DisplayRectangle.Width, this.DisplayRectangle.Top + this.DisplayRectangle.Height);
                            break;
                        case PlacementEnum.South:
                            triangleBorderTopPoint = new PointF(this.DisplayRectangle.Left + this.DisplayRectangle.Width / 2.0f, this.DisplayRectangle.Top);
                            e.Graphics.DrawLine(borderPen, this.DisplayRectangle.Left, this.DisplayRectangle.Top + this.DisplayRectangle.Height, triangleBorderTopPoint.X, triangleBorderTopPoint.Y);
                            e.Graphics.DrawLine(borderPen, triangleBorderTopPoint.X, triangleBorderTopPoint.Y, this.DisplayRectangle.Left + this.DisplayRectangle.Width, this.DisplayRectangle.Top + this.DisplayRectangle.Height);
                            e.Graphics.DrawLine(borderPen, this.DisplayRectangle.Left + this.DisplayRectangle.Width, this.DisplayRectangle.Top + this.DisplayRectangle.Height, this.DisplayRectangle.Left, this.DisplayRectangle.Top + this.DisplayRectangle.Height);
                            break;
                    }                    
                }
                catch
                {
                    throw;
                }
                finally
                {
                    borderPen.Dispose();
                }
            }
        }

        
    }
}
