using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using Ascend;
using Ascend.Windows.Forms;

namespace Lokeiv.Games.Ludo.Forms.Client
{
    public class HouseRender : GradientBackgroundRender
    {
        private int _posOffset = 2;
        private RectangleF _houseBounds;
        private Color _gradientWhiteHighColor;
        private HouseRender _whiteRender;

        public Color GradientWhiteHighColor
        {
            get { return _gradientWhiteHighColor; }
            set { _gradientWhiteHighColor = value; }
        }

        public RectangleF WhiteBounds
        {
            get { return _whiteRender.HouseBounds; }
            set { _whiteRender.HouseBounds = value; }
        }

        public RectangleF HouseBounds
        {
            get { return _houseBounds; }
            set { _houseBounds = value; this.DisplayRectangle = new Rectangle((int)value.X, (int)value.Y, (int)value.Width, (int)value.Height); }
        }

        public HouseRender(Lokeiv.Games.Ludo.Model.Color color)
        {
            this.Border = new Border(1);
            this.BorderColor = new BorderColor(System.Drawing.Color.Black);

            if (color != Lokeiv.Games.Ludo.Model.Color.White)
            {
                this._whiteRender = new HouseRender(Lokeiv.Games.Ludo.Model.Color.White);
            }

            // Select the color to draw this house with
            switch (color)
            {
                case Lokeiv.Games.Ludo.Model.Color.Red:
                    this.GradientHighColor = System.Drawing.Color.LightCoral;
                    this.GradientLowColor = System.Drawing.Color.DarkRed;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.Yellow:
                    this.GradientHighColor = System.Drawing.Color.LightYellow;
                    this.GradientLowColor = System.Drawing.Color.Yellow;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.Green:
                    this.GradientHighColor = System.Drawing.Color.LightGreen;
                    this.GradientLowColor = System.Drawing.Color.DarkGreen;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.Blue:
                    this.GradientHighColor = System.Drawing.Color.LightBlue;
                    this.GradientLowColor = System.Drawing.Color.DarkBlue;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.White:
                    this.GradientHighColor = System.Drawing.Color.White;
                    this.GradientLowColor = System.Drawing.Color.WhiteSmoke;
                    break;
            }  
        }

        public override void Render(System.Windows.Forms.PaintEventArgs e)
        {
            if (e == null)
            {
                throw new ArgumentNullException("e");
            }

            RectangleF bounds;
            float topOffset = (this._posOffset + base.Padding.Top + this.Border.Top);
            float leftPosition = (this.DisplayRectangle.Left + base.Padding.Left + this._posOffset);
            float topPosition = (this.DisplayRectangle.Top + this._posOffset + base.Padding.Top + this.Border.Top);
            float rightOffset = (this._posOffset + base.Padding.Right + this.Border.Right);
            float bottomOffset = (this._posOffset + base.Padding.Bottom + this.Border.Bottom);
            bounds = new RectangleF(leftPosition, topPosition, (this.DisplayRectangle.Width - (leftPosition + rightOffset)), (this.DisplayRectangle.Height - (topOffset + bottomOffset)));

            if (this.DisplayRectangle.Height > 0 && this.DisplayRectangle.Width > 0)
            {
                base.Render(e);

                // Draw the white part of the house
                if (this._whiteRender != null && !this._whiteRender.HouseBounds.IsEmpty)
                {
                    this._whiteRender.Render(e);
                }
            }
        }
    }
}
