using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Lokeiv.Games.Ludo.Model;
using Lokeiv.Games.Ludo.Forms.InternetGame.LudoService;

namespace Lokeiv.Games.Ludo.Forms.InternetGame
{
    public partial class LudoTable : Form
    {
        private LudoRoom _containingRoom = null;
        private Table _table = null;
        private Client _client = null;
        private ILudoService _ludoService;
        private ServiceCallback.LudoServiceCallback _ludoServiceCallback;

        public LudoTable(LudoRoom room, Table table, Client client, ILudoService ludoService, ServiceCallback.LudoServiceCallback ludoServiceCallback)
        {
            InitializeComponent();
            this._containingRoom = room;
            this._table = table;
            this._client = client;
            this._ludoService = ludoService;
            this._ludoServiceCallback = ludoServiceCallback;
            this._ludoServiceCallback.ClientEnteredTable += new EventHandler<ServiceCallback.LudoServiceTableEventArgs>(TableChanges);
            this._ludoServiceCallback.ClientLeftTable += new EventHandler<ServiceCallback.LudoServiceTableEventArgs>(TableChanges);
            this._ludoServiceCallback.ClientSatDown += new EventHandler<ServiceCallback.LudoServiceTableEventArgs>(TableChanges);
            this._ludoServiceCallback.ClientRaisedUp += new EventHandler<ServiceCallback.LudoServiceTableEventArgs>(TableChanges);
            this.UpdateView();
        }

        private void UpdateView()
        {
            this.SetButtonState(this.redButton, this._table.RedPlayer);
            this.SetButtonState(this.yellowButton, this._table.YellowPlayer);
            this.SetButtonState(this.greenButton, this._table.GreenPlayer);
            this.SetButtonState(this.blueButton, this._table.BluePlayer);
        }

        private void SetButtonState(Button button, Client clientAtPosition)
        {
            if (clientAtPosition == null)
            {
                button.Text = "Sit Down";
                button.Enabled = true;
            }
            else if (clientAtPosition.NickName == this._client.NickName)
            {
                button.Text = "Raise Up";
                button.Enabled = true;
            }
            else
            {
                button.Text = clientAtPosition.NickName;
                button.Enabled = false;
            }
        }

        private void HandlePlayerButtonClick(Lokeiv.Games.Ludo.Model.Color color, Button button)
        {
            switch (color)
            {
                case Model.Color.Red:
                    if (this._table.RedPlayer != null && this._table.RedPlayer.NickName == this._client.NickName)
                    {
                        this._ludoService.RaiseUp(this._client.NickName, this._table.Name);
                    }
                    else if (this._table.RedPlayer == null)
                    {
                        this._ludoService.SitDown(this._client.NickName, this._table.Name, color);
                    }
                    break;
                case Model.Color.Yellow:
                    if (this._table.YellowPlayer != null && this._table.YellowPlayer.NickName == this._client.NickName)
                    {
                        this._ludoService.RaiseUp(this._client.NickName, this._table.Name);
                    }
                    else if (this._table.YellowPlayer == null)
                    {
                        this._ludoService.SitDown(this._client.NickName, this._table.Name, color);
                    }
                    break;
                case Model.Color.Green:
                    if (this._table.GreenPlayer != null && this._table.GreenPlayer.NickName == this._client.NickName)
                    {
                        this._ludoService.RaiseUp(this._client.NickName, this._table.Name);
                    }
                    else if (this._table.GreenPlayer == null)
                    {
                        this._ludoService.SitDown(this._client.NickName, this._table.Name, color);
                    }
                    break;
                case Model.Color.Blue:
                    if (this._table.BluePlayer != null && this._table.BluePlayer.NickName == this._client.NickName)
                    {
                        this._ludoService.RaiseUp(this._client.NickName, this._table.Name);
                    }
                    else if (this._table.BluePlayer == null)
                    {
                        this._ludoService.SitDown(this._client.NickName, this._table.Name, color);
                    }
                    break;
            }

            this._table = this._ludoService.RefreshTable(this._table.Name);
            this.UpdateView();
        }

        void TableChanges(object sender, ServiceCallback.LudoServiceTableEventArgs e)
        {
            if (e.Table.Name == this._table.Name)
            {
                this.UpdateView();
            }
        }

        private void LudoTable_FormClosed(object sender, FormClosedEventArgs e)
        {
            this._containingRoom.TableFormsOpen.Remove(this);

            if (this._containingRoom.TableFormsOpen.Count == 0 &&
                !this._containingRoom.Visible)
            {
                // No other window is open, so exit the application.
                Application.Exit();
            }
        }

        private void redButton_Click(object sender, EventArgs e)
        {
            HandlePlayerButtonClick(Model.Color.Red, sender as Button);
        }

        private void yeloowButton_Click(object sender, EventArgs e)
        {
            HandlePlayerButtonClick(Model.Color.Yellow, sender as Button);
        }

        private void greenButton_Click(object sender, EventArgs e)
        {
            HandlePlayerButtonClick(Model.Color.Green, sender as Button);
        }

        private void blueButton_Click(object sender, EventArgs e)
        {
            HandlePlayerButtonClick(Model.Color.Blue, sender as Button);
        }
    }
}