using System;
using System.Collections.Generic;
using System.Text;

namespace Lokeiv.Games.Ludo.Model.ComputerPlayer
{
    public class MoveStrategyFactory
    {
        #region Singleton Pattern
        private static MoveStrategyFactory _instance = null;

        private MoveStrategyFactory()
        {

        }

        public static MoveStrategyFactory Instance
        {
            get
            {
                if (_instance == null)
                    _instance = new MoveStrategyFactory();

                return _instance;
            }
        }
        #endregion

        /// <summary>
        /// Selects a movement from the possible movements in <paramref name="movablePieces"/>, using a defined strategy.
        /// </summary>
        /// <param name="movablePieces"><see cref="List<MoveablePiece>"/> of possible moves.</param>
        /// <returns><see cref="MoveablePiece"/> selected by the player's strategy.</returns>
        public MoveablePiece SelectPieceToMove(List<MoveablePiece> movablePieces)
        {
            // Do what's the best for the player
            CompositeBestForPlayer strategy = new CompositeBestForPlayer();

            foreach (MoveablePiece movable in movablePieces)
                strategy.AddCorrectMoveStrategy(movable);

            return strategy.GetBestMove();
        }
    }
}
