using System;
using System.Collections.Generic;
using System.Text;
using Lokeiv.Games.Dice;
using System.Xml.Serialization;

namespace Lokeiv.Games.Ludo.Model
{
    public class Response : IXmlSerializable
    {

        #region Properties
        private DieValue _dieValue = null;
        public DieValue DieValue
        {
            get { return this._dieValue; }
            set { this._dieValue = value; }
        }

        private List<MoveablePiece> _movablePieces = new List<MoveablePiece>();
        public List<MoveablePiece> MovablePieces
        {
            get { return this._movablePieces; }
            set { this._movablePieces = value; }
        }

        private Player _playersTurn = null;
        public Player PlayersTurn
        {
            get { return this._playersTurn; }
            set { this._playersTurn = value; }
        }

        private Player _playerWon = null;
        public Player PlayerWon
        {
            get { return this._playerWon; }
            set { this._playerWon = value; }
        }

        private Action _nextAction = Action.Nothing;
        public Action NextAction
        {
            get { return this._nextAction; }
            set { this._nextAction = value; }
        }
        #endregion

        #region Contructors
        public Response() { }

        public Response(DieValue dieValue, Player playersTurn, Action nextAction)
        {
            this._dieValue = dieValue;
            this._playersTurn = playersTurn;
            this._nextAction = nextAction;
        }

        public Response(Player playerWon, Action nextAction)
        {
            this._playerWon = playerWon;
            this._nextAction = nextAction;
        }
        #endregion

        #region IXmlSerializable
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(System.Xml.XmlReader reader)
        {
            throw new NotImplementedException();
        }

        public static Response ReadXml(System.Xml.XmlReader reader, Board board, PlayerCollection players)
        {
            reader.MoveToContent();
            if (reader.IsEmptyElement)
                return null;

            Response response = new Response();
            string nextAction = reader.GetAttribute("NextAction");
            response.NextAction = (Lokeiv.Games.Ludo.Action)Enum.Parse(typeof(Lokeiv.Games.Ludo.Action), nextAction, true);

            while (reader.Read())
            {
                if (reader.NodeType == System.Xml.XmlNodeType.Element)
                {
                    switch(reader.Name.ToLower())
                    {
                        case "dievalue":
                            response.DieValue = new DieValue(null);
                            response.DieValue.ReadXml(reader.ReadSubtree());                            
                            reader.Skip();                            
                            break;
                        case "movablepieces":
                            response.MovablePieces = MoveablePiece.ReadCollectionFromXml(reader.ReadSubtree(), board);
                            reader.Skip();
                            break;
                        case "playersturn":
                            response.PlayersTurn = Player.ReadXml(reader.ReadSubtree(), players);
                            reader.Skip();
                            break;
                        case "playerwon":
                            response.PlayerWon = Player.ReadXml(reader.ReadSubtree(), players);
                            reader.Skip();
                            break;
                    }
                }
            }

            return response;
        }        

        public void WriteXml(System.Xml.XmlWriter writer)
        {
            // Write attributes
            writer.WriteAttributeString("NextAction", this.NextAction.ToString());

            // Write DieValue
            writer.WriteStartElement("DieValue");
            if (this.DieValue != null) this.DieValue.WriteXml(writer);
            writer.WriteEndElement();

            // Write MovablePieces
            writer.WriteStartElement("MovablePieces");

            foreach (MoveablePiece movablePiece in this.MovablePieces)
            {
                writer.WriteStartElement("MovablePiece");
                movablePiece.WriteXml(writer);
                writer.WriteEndElement();
            }

            writer.WriteEndElement();

            // Write PlayersTurn
            writer.WriteStartElement("PlayersTurn");
            if (this.PlayersTurn != null) this.PlayersTurn.WriteXmlReference(writer);
            writer.WriteEndElement();

            // Write PlayerWon
            writer.WriteStartElement("PlayerWon");
            if (this.PlayerWon != null) this.PlayerWon.WriteXmlReference(writer);
            writer.WriteEndElement();
        }
        #endregion
    }
}
