using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;

namespace Lokeiv.Games.Ludo.Model
{
    public class Square : IXmlSerializable
    {   

        #region Properties
        private Color _color = Color.White;
        public Color Color
        {
            get { return this._color; }            
        }

        private int _number = 0;
        public int Number
        {
            get { return this._number; }            
        }

        private Boolean _isCrossing = false;  // crossing is the square outside of the house
        public Boolean IsCrossing
        {
            get { return this._isCrossing; }            
        }

        private Boolean _isHouse = false;
        public Boolean IsHouse
        {
            get { return this._isHouse; }            
        }

        private Boolean _isGoal = false;
        public Boolean IsGoal
        {
            get { return this._isGoal; }
        }

        private PieceCollection _pieces = new PieceCollection();
        public PieceCollection Pieces
        {
            get { return this._pieces; }
        }

        private Square _nextSquare = null;
        public Square NextSquare
        {
            get { return this._nextSquare; }
        }

        private Square _alternateSquare = null;
        public Square AlternateSquare
        {
            get { return this._alternateSquare; }
        }

        private Block _block = null;
        public Block Block
        {
            get { return this._block; }
        }

#endregion

        #region Constructors
        /// <summary>
        /// Non-public default constructor for deserialization.
        /// </summary>
        private Square()
        {

        }

        internal Square(Color color, int number) : this()
        {
            this._color = color;
            this._number = number;
        }
        #endregion

        #region Internal Methods
        internal void SetColor(Color color)
        {
            this._color = color;
        }

        internal void SetNumber(int number)
        {
            this._number = number;
        }

        internal void SetCrossing(Boolean isCrossing)
        {
            this._isCrossing = isCrossing;
        }

        internal void SetHouse(Boolean isHouse)
        {
            this._isHouse = isHouse;
        }

        internal void SetGoal(Boolean isGoal)
        {
            this._isGoal = isGoal;
        }

        internal void SetNextSquare(Square square)
        {
            this._nextSquare = square;
        }

        internal void SetAlternateSquare(Square square)
        {
            this._alternateSquare = square;
        }

        internal void SetBlock(Block block)
        {
            this._block = block;
        }
        #endregion

        #region IXmlSerializable
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(System.Xml.XmlReader reader)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Reads serialized square content into an existing square object.
        /// This content may be pieces on the square and a block that is formed on the square.
        /// </summary>
        /// <param name="reader"><see cref="System.Xml.XmlReader"/></param>
        /// <param name="board"><see cref="Board">containing all squares</see></param>
        public void ReadXml(System.Xml.XmlReader reader, Board board)
        {
            reader.MoveToContent();

            while (reader.Read())
            {
                if (reader.NodeType == System.Xml.XmlNodeType.Element)
                {
                    switch(reader.Name.ToLower())
                    {
                        case "pieces":
                            this.Pieces.ReadXml(reader.ReadSubtree(), board);
                            reader.Skip();
                            break;
                        case "block":
                            Block block = new Block(Model.Color.White);
                            block.ReadXml(reader.ReadSubtree());
                            if (block.Color != Model.Color.White)
                                this._block = block;
                            reader.Skip();
                            break;
                    }
                }
            }
        }

        /// <summary>
        /// Reads a square reference and nothing else.
        /// </summary>
        /// <param name="reader"><see cref="System.Xml.XmlReader"/></param>
        /// <param name="board"><see cref="Board">containing all squares</see></param>
        /// <returns><see cref="Square"/></returns>
        public static Square ReadXmlReference(System.Xml.XmlReader reader, Board board)
        {
            reader.MoveToContent();
            string number = reader.GetAttribute("Number");
            return board.Squares[int.Parse(number)];
        }        

        public void WriteXml(System.Xml.XmlWriter writer)
        {
            this.WriteXmlReference(writer);
            
            // Write Pieces
            if (this.Pieces.Count > 0)
            {
                writer.WriteStartElement("Pieces");

                foreach (Piece piece in this.Pieces)
                {
                    writer.WriteStartElement("Piece");
                    piece.WriteXmlReference(writer);
                    writer.WriteEndElement();
                }
                    
                writer.WriteEndElement();
            }

            // Write Block
            if (this.Block != null)
            {
                writer.WriteStartElement("Block");
                this.Block.WriteXml(writer);
                writer.WriteEndElement();
            }
        }

        public void WriteXmlReference(System.Xml.XmlWriter writer)
        {
            writer.WriteAttributeString("Number", this.Number.ToString());
        }
        #endregion
    }
}
