﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Data;
using System.Windows.Media.Imaging;
using Lokeiv.Games.Ludo.Model;

namespace Ludo.WP7.Client.Utils
{
    public class LudoColorToLinearGradientBrushConverter : IValueConverter
    {

        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            System.Windows.Media.Color color1 = Colors.White;
            System.Windows.Media.Color color2 = Colors.Black;
            bool reduceOpacity = false;

            if (parameter is string && parameter.ToString().ToLower() == "semitransparent")
                reduceOpacity = true;

            if (targetType == typeof(Brush))
            {
                if (value is Lokeiv.Games.Ludo.Model.Color)
                {
                    Lokeiv.Games.Ludo.Model.Color ludoColor = (Lokeiv.Games.Ludo.Model.Color)value;

                    switch (ludoColor)
                    {
                        case Lokeiv.Games.Ludo.Model.Color.Red:
                            color1 = System.Windows.Media.Color.FromArgb(255, 240, 128, 128);  // LightCoral
                            color2 = System.Windows.Media.Color.FromArgb(255, 139, 0, 0);  // DarkRed
                            break;
                        case Lokeiv.Games.Ludo.Model.Color.Yellow:
                            color1 = System.Windows.Media.Color.FromArgb(255, 255, 255, 224);  // LightYellow
                            color2 = System.Windows.Media.Color.FromArgb(255, 255, 255, 0);  // Yellow
                            break;
                        case Lokeiv.Games.Ludo.Model.Color.Green:
                            color1 = System.Windows.Media.Color.FromArgb(255, 144, 238, 144);  // LightGreen
                            color2 = System.Windows.Media.Color.FromArgb(255, 0, 100, 0);  // DarkGreen
                            break;
                        case Lokeiv.Games.Ludo.Model.Color.Blue:
                            color1 = System.Windows.Media.Color.FromArgb(255, 173, 216, 230);  // LightBlue
                            color2 = System.Windows.Media.Color.FromArgb(255, 0, 0, 139);  // DarkBlue
                            break;
                        default:
                            color1 = System.Windows.Media.Colors.White;  // White
                            color2 = System.Windows.Media.Color.FromArgb(255, 245, 245, 245);  // WhiteSmoke
                            break;
                    }
                }

                if (reduceOpacity)
                {
                    return CreateLinearGradientBrush(ReduceOpacity(color1), ReduceOpacity(color2));
                }
                else
                {
                    return CreateLinearGradientBrush(color1, color2);
                }
            }
            else
            {                
                return CreateLinearGradientBrush(color1, color2);
            }
        }

        private Brush CreateLinearGradientBrush(System.Windows.Media.Color color1, System.Windows.Media.Color color2)
        {
            GradientStopCollection gsc = new GradientStopCollection();
            gsc.Add(new GradientStop { Color = color1, Offset = 0 });
            gsc.Add(new GradientStop { Color = color2, Offset = 1 });
            
            return new LinearGradientBrush(gsc, 45);
        }

        private System.Windows.Media.Color ReduceOpacity(System.Windows.Media.Color color)
        {
            return System.Windows.Media.Color.FromArgb(128, color.R, color.G, color.B);
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
