﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Xml.Serialization;

namespace Ludo.WP7.Client.ViewModels
{
    public class PieceViewModel : INotifyPropertyChanged, IXmlSerializable
    {

        #region Properties

        private Lokeiv.Games.Ludo.Model.Color _pieceColor = Lokeiv.Games.Ludo.Model.Color.White;
        public Lokeiv.Games.Ludo.Model.Color PieceColor
        {
            get { return _pieceColor; }
            set 
            {
                if (_pieceColor != value)
                {
                    _pieceColor = value;
                    NotifyPropertyChanged("PieceColor");
                }
            }
        }

        private double _left = 0;
        public double Left
        {
            get { return _left; }
            set
            {
                if (_left != value)
                {
                    _left = value;
                    NotifyPropertyChanged("Left");
                }
            }
        }

        private double _top = 0;
        public double Top
        {
            get { return _top; }
            set
            {
                if (_top != value)
                {
                    _top = value;
                    NotifyPropertyChanged("Top");
                }
            }
        }

        private int _pieceCount = 1;
        public int PieceCount
        {
            get { return _pieceCount; }
            set
            {
                if (_pieceCount != value)
                {
                    _pieceCount = value;
                    NotifyPropertyChanged("PieceCount");
                    NotifyPropertyChanged("PieceCountString");
                }
            }
        }

        public string PieceCountString
        {
            get
            {
                if (PieceCount < 2)
                    return string.Empty;

                return PieceCount.ToString();
            }
        }

        private bool _isPieceInGame = true;
        public bool IsPieceInGame
        {
            get { return _isPieceInGame; }
            set
            {
                if (_isPieceInGame != value)
                {
                    _isPieceInGame = value;
                    NotifyPropertyChanged("IsPieceInGame");
                    NotifyPropertyChanged("Visibility");
                }
            }
        }

        public Visibility Visibility
        {
            get { return IsPieceInGame ? System.Windows.Visibility.Visible : System.Windows.Visibility.Collapsed; }
        }

        #endregion  // Properties

        #region Constructors
        public PieceViewModel(Lokeiv.Games.Ludo.Model.Color pieceColor)
        {
            this.PieceColor = pieceColor;
        }
        #endregion  // Constructors

        #region Methods

        public void SetPiecePosition(Rect pieceBounds)
        {
            this.Left = pieceBounds.Left;
            this.Top = pieceBounds.Top;
        }

        #endregion  // Methods

        #region INotifyPropertyChanged
        /// <summary>
        /// Event to notify the view when a property has changed.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Fires the <see cref="PropertyChanged"/> event for the given <paramref name="propertyName"/>.
        /// </summary>
        /// <param name="propertyName">Name of property that has changed.</param>
        private void NotifyPropertyChanged(String propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion  // INotifyPropertyChanged

        #region IXmlSerializable
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(System.Xml.XmlReader reader)
        {
            reader.MoveToContent();

            string color = reader.GetAttribute("Color");
            if (!string.IsNullOrEmpty(color)) this.PieceColor = (Lokeiv.Games.Ludo.Model.Color)Enum.Parse(typeof(Lokeiv.Games.Ludo.Model.Color), color, true);

            string left = reader.GetAttribute("Left");
            this.Left = double.Parse(left);
            
            string top = reader.GetAttribute("Top");
            this.Top = double.Parse(top);

            string pieceCount = reader.GetAttribute("PieceCount");
            this.PieceCount = int.Parse(pieceCount);

            string isPieceInGame = reader.GetAttribute("IsPieceInGame");
            this.IsPieceInGame = bool.Parse(isPieceInGame);
        }

        public void WriteXml(System.Xml.XmlWriter writer)
        {
            writer.WriteAttributeString("Color", this.PieceColor.ToString());
            writer.WriteAttributeString("Left", this.Left.ToString());
            writer.WriteAttributeString("Top", this.Top.ToString());
            writer.WriteAttributeString("PieceCount", this.PieceCount.ToString());
            writer.WriteAttributeString("IsPieceInGame", this.IsPieceInGame.ToString());
        }
        #endregion
    }
}
