using System;
using System.Collections.Generic;
using System.Text;

namespace Lokeiv.Games.Dice.Generic
{
    public abstract class Die<T>
    {
        protected DieValue<T> _value = null;
        protected Random _randomizer = null;

        # region Public properties

        public DieValue<T> Value
        {
            get { return this._value; }
            set { this._value = value; }
        }

        #endregion

        #region Public events

        public delegate void DieRolledEventHandler(object die, DieRolledEventArgs<T> e);

        public event DieRolledEventHandler DieRolled;

        protected void OnDieRolled(object die, DieRolledEventArgs<T> e)
        {
            if (this.DieRolled != null)
            {
                this.DieRolled(this, e);
            }
        }

        #endregion

        #region Constructors

        public Die()
        {            
            this._randomizer = new Random();
        }

        #endregion

        protected abstract DieValue<T> GetNewRandomValue();

        public DieValue<T> Roll()
        {
            this._value = this.GetNewRandomValue();
            this.OnDieRolled(this, new DieRolledEventArgs<T>(this._value));
            return this._value;
        }

    }
}
