﻿using System;

using Android.App;
using Android.Content;
using Android.Runtime;
using Android.Views;
using Android.Widget;
using Android.OS;
using GameFx.ViewModels;
using System.ComponentModel;

namespace GameFx.Android.App
{
    [Activity(Label = "Lindgren Multiplayer Game Framework - Lobby", MainLauncher = true, Icon = "@drawable/icon")]
    public class LobbyActivity : Activity
    {
        private LobbyViewModel ViewModel { get; set; }
        
        protected override void OnCreate(Bundle bundle)
        {
            base.OnCreate(bundle);
            SetContentView(Resource.Layout.Main);

            if (this.ViewModel == null)
            {
                this.ViewModel = new LobbyViewModel(false);
                this.ViewModel.PropertyChanged += new PropertyChangedEventHandler(ViewModel_PropertyChanged);
                this.ViewModel.Enter.CanExecuteChanged += new EventHandler(Enter_CanExecuteChanged);
                this.ViewModel.Chat.CanExecuteChanged += new EventHandler(Chat_CanExecuteChanged);
            }

            Button enterButton = FindViewById<Button>(Resource.Id.EnterButton);
            enterButton.Click += delegate { if (this.ViewModel.Enter.CanExecute(null)) this.ViewModel.Enter.Execute(null); };

            Button chatButton = FindViewById<Button>(Resource.Id.SendButton);
            chatButton.Click += delegate { if (this.ViewModel.Chat.CanExecute(null)) this.ViewModel.Chat.Execute(null); };

            EditText nickNameEditText = this.FindViewById<EditText>(Resource.Id.NickNameEditText);
            nickNameEditText.TextChanged += delegate
            {
                if (nickNameEditText.Text != this.ViewModel.NickName)
                    this.ViewModel.NickName = nickNameEditText.Text;
            };

            EditText messageEditText = this.FindViewById<EditText>(Resource.Id.MessageEditText);
            messageEditText.TextChanged += delegate
            {
                if (messageEditText.Text != this.ViewModel.ChatMessage)
                    this.ViewModel.ChatMessage = messageEditText.Text;
            };
        }

        void Chat_CanExecuteChanged(object sender, EventArgs e)
        {
            this.FindViewById<Button>(Resource.Id.SendButton).Enabled = this.ViewModel.Chat.CanExecute(null);
        }

        void Enter_CanExecuteChanged(object sender, EventArgs e)
        {
            this.FindViewById<Button>(Resource.Id.EnterButton).Enabled = this.ViewModel.Enter.CanExecute(null);
        }

        private void ViewModel_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case "NickName":
                    this.FindViewById<EditText>(Resource.Id.NickNameEditText).Text = this.ViewModel.NickName;
                    break;
                case "Messages":
                    this.FindViewById<TextView>(Resource.Id.MessagesEditText).Text = this.ViewModel.Messages;
                    break;
                case "ChatMessage":
                    this.FindViewById<EditText>(Resource.Id.MessageEditText).Text = this.ViewModel.ChatMessage;
                    break;
                case "Clients":
                    this.UpdateListOfClients();
                    break;
                default:
                    throw new NotImplementedException(string.Format("ViewModel_PropertyChanged not implemented for property {0}.", e.PropertyName));
            }
        }

        private void UpdateListOfClients()
        {

        }
    }
}

