﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using GameFx.ViewModels;
using System.Threading;
using System.Collections.Specialized;

namespace GameFx.WindowsForms.App
{
    public partial class LobbyForm : Form
    {
        private bool IsDesignMode { get; set; }
        private LobbyViewModel ViewModel { get; set; }
        private SynchronizationContext _syncContext;

        public LobbyForm()
        {
            InitializeComponent();
            this.IsDesignMode = (LicenseManager.UsageMode == LicenseUsageMode.Designtime);
            this._syncContext = SynchronizationContext.Current;
            this.ViewModel = (LobbyViewModel)this.viewModelBindingSource.Current;
            this.ViewModel.PropertyChanged += new PropertyChangedEventHandler(ViewModel_PropertyChanged);
            this.ViewModel.Clients.CollectionChanged +=new NotifyCollectionChangedEventHandler(Clients_CollectionChanged);
            this.nickNameTextBox.DataBindings[0].DataSourceUpdateMode = DataSourceUpdateMode.OnPropertyChanged;
            this.chatTextBox.DataBindings[0].DataSourceUpdateMode = DataSourceUpdateMode.OnPropertyChanged;
        }

        /// <summary>
        /// Since the ListView control doesn't support databinding, the view must be manually updated.
        /// </summary>
        private void UpdateClientsList()
        {
            this.clientsListView.BeginUpdate();

            try
            {
                this.clientsListView.Items.Clear();

                foreach (GameFx.Entities.Client client in this.ViewModel.Clients)
                {
                    this.clientsListView.Items.Add(client.NickName);
                }
            }
            finally
            {
                this.clientsListView.EndUpdate();
            }
        }

        private void ViewModel_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Clients")
            {
                this.ViewModel.Clients.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(Clients_CollectionChanged);
                this.UpdateClientsList();
            }
        }

        private void Clients_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            this.UpdateClientsList();
        }

        private void Form1_Load(object sender, EventArgs e)
        {

        }

        private void chatTextBox_KeyUp(object sender, KeyEventArgs e)
        {
            if (!e.Control && !e.Shift && (e.KeyCode == Keys.Enter || e.KeyCode == Keys.Return))
            {
                e.Handled = true;
                this.ViewModel.ChatMessage = this.ViewModel.ChatMessage.Substring(0, this.ViewModel.ChatMessage.Length - 2);  // Remove the last line break
                this.ViewModel.Chat.Execute(null);
            }
        }

        private void nickNameTextBox_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter || e.KeyCode == Keys.Return)
            {
                e.Handled = true;

                if (this.ViewModel.Enter.CanExecute(null))
                    this.ViewModel.Enter.Execute(null);
            }
        }

    }
}
