﻿using System.ComponentModel;
using System.Drawing.Design;
using System.Linq;
using System.Reflection;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Windows.Input;

namespace GameFx.WindowsForms.Client.MVVM
{
	public class CommandEditor : UITypeEditor
	{
		public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context)
		{
			return UITypeEditorEditStyle.DropDown;
		}

		public override object EditValue(ITypeDescriptorContext context, System.IServiceProvider provider, object value)
		{
			var service = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;
			if (null != service)
			{
				var commandExtenderProvider = GetCommandExtenderProvider(context);

				if (null != commandExtenderProvider && 
					null != commandExtenderProvider.ViewModelSource &&
					null != commandExtenderProvider.ViewModelSource.Current )
				{
					var properties = GetCommandsFromSource(commandExtenderProvider.ViewModelSource);
					var listBox = new ListBox();
					foreach( var property in properties )
					{
						listBox.Items.Add(property.Name);
					}

					listBox.SelectedIndexChanged += (s, e) => service.CloseDropDown();

					service.DropDownControl(listBox);

					
					return listBox.SelectedItem;
				}
			}

			return null;
		}

		private PropertyInfo[] GetCommandsFromSource(BindingSource source)
		{
			if( null == source.Current )
			{
				return new PropertyInfo[0];
			}
			var type = source.Current.GetType();
			var properties = type.GetProperties();
			var query = from p in properties
			            where p.PropertyType.Equals(typeof (ICommand))
			            select p;

			return query.ToArray();
		}

		private CommandExtenderProvider	GetCommandExtenderProvider(ITypeDescriptorContext context)
		{
			foreach( var component in context.Container.Components )
			{
				if( component is CommandExtenderProvider )
				{
					return component as CommandExtenderProvider;
				}
				
			}
			return null;
		}
	}
}