﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GameFx.Entities
{
    public partial class Client : EntityBase, IComparable, IComparable<Client>, IEquatable<Client>
    {
        public ClientToken Token { get; private set; }
        public string NickName { get; set; }

        private Client()
        {
            this.Token = new ClientToken();
        }

        public Client(string nickName)
            : this()
        {
            this.NickName = nickName;
        }

        #region IXmlSerializable
        public override void ReadXml(System.Xml.XmlReader reader)
        {
            reader.MoveToContent();

            // Token
            reader.ReadToFollowing("Token");
            this.Token.ReadXml(reader);

            // NickName
            reader.ReadToFollowing("NickName");
            this.NickName = reader.ReadInnerXml();
            reader.Skip();
        }

        public static Client CreateFromXml(System.Xml.XmlReader reader)
        {
            Client client = new Client();
            client.ReadXml(reader);

            return client;
        }

        public override void WriteXml(System.Xml.XmlWriter writer)
        {
            // Token
            writer.WriteStartElement("Token");
            this.Token.WriteXml(writer);
            writer.WriteEndElement();

            // NickName
            writer.WriteStartElement("NickName");
            writer.WriteString(this.NickName);
            writer.WriteEndElement();
        }
        #endregion  // IXmlSerializable

        #region IComparable
        public int CompareTo(object obj)
        {
            if (obj == null || !(obj is Client))
                return -1;

            return this.CompareTo((Client)obj);
        }
        #endregion  // IComparable

        #region IComparable<Client>
        public int CompareTo(Client other)
        {
            if (other == null)
                return -1;

            return this.Token.Id.CompareTo(other.Token.Id);
        }
        #endregion  // IComparable<Client>

        #region IEquatable<Client>
        public bool Equals(Client other)
        {
            if (other == null)
                return false;

            return this.Token.Id == other.Token.Id;
        }
        #endregion  // IEquatable<Client>
    }
}
