﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GameFx.Entities
{
    public class ClientToken : EntityBase
    {
        public Guid Id { get; private set; }
        public byte Version { get; private set; }
        public Platform Platform { get; private set; }

        public ClientToken()
        {
            this.Id = Guid.NewGuid();
            this.Version = 1;
            this.Platform = Entities.Platform.Undefined;
            this.SetPlatform();
        }

        private void SetPlatform()
        {
#if WPF
            this.Platform = Entities.Platform.Wpf;
#endif

#if WINDOWS_PHONE
            this.Platform = Entities.Platform.WindowsPhone;
#endif

#if WINDOWS_FORMS
            this.Platform = Entities.Platform.WindowsForms;
#endif

#if ANDROID
            this.Platform = Entities.Platform.Android;
#endif

#if HTML
            this.Platform = Entities.Platform.Html;
#endif
        }
        
        #region IXmlSerializable
        public override void ReadXml(System.Xml.XmlReader reader)
        {
            reader.MoveToContent();
            this.Id = new Guid(reader.GetAttribute("Id"));
            this.Version = byte.Parse(reader.GetAttribute("Version"));
            this.Platform = (Platform)System.Enum.Parse(typeof(Platform), reader.GetAttribute("Platform"), true);
            reader.Skip();
        }

        public static ClientToken CreateFromXml(System.Xml.XmlReader reader)
        {
            ClientToken token = new ClientToken();
            token.ReadXml(reader);
            
            return token;
        }

        public override void WriteXml(System.Xml.XmlWriter writer)
        {
            // Id
            writer.WriteStartAttribute("Id");
            writer.WriteValue(this.Id.ToString());
            writer.WriteEndAttribute();

            // Version
            writer.WriteStartAttribute("Version");
            writer.WriteValue(this.Version);
            writer.WriteEndAttribute();

            // Platform
            writer.WriteStartAttribute("Platform");
            writer.WriteValue(this.Platform.ToString());
            writer.WriteEndAttribute();
        }
        #endregion  // IXmlSerializable
    }
}
