﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CommonClient.GameClient.GameService;
using System.Threading;
using System.ServiceModel;

namespace CommonClient.GameClient
{
    public class GameServiceCallback : IGameServiceCallback
    {
        #region Fields and properties
        private SynchronizationContext _syncContext;
        private Client _gameClient;
        private IMainViewModel _mainViewModel;
        
        #endregion  // Fields and properties

        #region Constructors
        public GameServiceCallback(Client client, IMainViewModel mainViewModel, SynchronizationContext syncContext)
        {
            this._syncContext = syncContext;
            this._gameClient = client;
            this._mainViewModel = mainViewModel;
        }
        #endregion  // Constructors

        #region Methods

        #region IGameServiceCallback
        public IAsyncResult BeginClientEntered(Client client, AsyncCallback callback, object asyncState)
        {
            if (asyncState is Guid && (Guid)asyncState != this._gameClient.Id)
            {
                this.ClientEntered(client);
            }

            return null;
        }

        public void ClientEntered(Client client)
        {
            SendOrPostCallback postCallback = delegate
            {
                this._mainViewModel.AppendMessage("Client entered: " + client.NickName);
                this._mainViewModel.ClientEntered();
            };

            this.MarshalToUiThread(postCallback);
        }

        public void EndClientEntered(IAsyncResult result)
        {

        }

        public void ClientSaid(Client client, string chatMessage)
        {
            SendOrPostCallback postCallback = delegate
            {
                this._mainViewModel.AppendMessage(client.NickName + " says: " + chatMessage);
            };

            this.MarshalToUiThread(postCallback);
        }

        public IAsyncResult BeginClientSaid(Client client, string chatMessage, AsyncCallback callback, object asyncState)
        {
            this.ClientSaid(client, chatMessage);
            return null;
        }

        public void EndClientSaid(IAsyncResult result)
        {

        }
        #endregion  // IGameServiceCallback

        #region Other methods
        /// <summary>
        /// Marshals any delegated content to the UI thread.
        /// </summary>
        /// <param name="contentToUI">SendOrPostCallback delegate</param>
        private void MarshalToUiThread(SendOrPostCallback contentToUI)
        {
            // Deliver delegated content to UI thread
            this._syncContext.Post(contentToUI, null);
        }
        #endregion  // Other methods

        #endregion  // Methods
    }
}
