﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Threading;
using System.ServiceModel;
using System.Windows.Input;
using CommonClient.GameClient.GameService;
using CommonClient.GameClient;

namespace CommonClient.ViewModels
{
    public class MainViewModel : INotifyPropertyChanged, IMainViewModel
    {
        #region Properties
        private Client _gameClient;
        private GameServiceClient _gameService;
        private GameServiceCallback _gameServiceCallback;
        private bool _isEntering, _hasEntered;

        private string _nickName;
        public string NickName
        {
            get
            {
                return this._nickName;
            }
            set
            {
                if (this._nickName != value)
                {
                    this._nickName = value;
                    this.NotifyPropertyChanged("NickName");
                }
            }
        }

        private string _messages;
        public string Messages
        {
            get
            {
                return this._messages;
            }
            set
            {
                if (this._messages != value)
                {
                    this._messages = value;
                    this.NotifyPropertyChanged("Messages");
                }
            }
        }

        private string _chatMessage;
        public string ChatMessage
        {
            get
            {
                return this._chatMessage;
            }
            set
            {
                if (this._chatMessage != value)
                {
                    this._chatMessage = value;
                    this.NotifyPropertyChanged("ChatMessage");
                }
            }
        }

        public ICommand Enter { get; private set; }
        private bool CanEnter
        {
            get
            {
                return !(this._isEntering || this._hasEntered);
            }
        }

        public ICommand Chat { get; private set; }
        private bool CanChat
        {
            get
            {
                return !string.IsNullOrEmpty(this.ChatMessage);
            }
        }
        #endregion  // Properties

        #region Constructors
        public MainViewModel(bool isDesignMode)
        {
            if (!isDesignMode)
            {
#if WINDOWS_PHONE || ANDROID || WINDOWS_FORMS
                this.Enter = new RelayCommand(param => this.OnEnter(), param => this.CanEnter);
                this.Chat = new RelayCommand(param => this.OnChat(), param => this.CanChat);
#else
                this.Enter = new RoutedCommand();
                this.Chat = new RoutedCommand();
#endif
                SynchronizationContext syncContext = SynchronizationContext.Current;
                this._gameServiceCallback = new GameServiceCallback(this._gameClient, this, syncContext);
                InstanceContext instanceContext = new InstanceContext(this._gameServiceCallback);
                this._gameService = new GameServiceClient(instanceContext);
            }
        }
        #endregion  // Constructors

        #region Methods

        #region Commands
        private void OnEnter()
        {
            try
            {
                this._isEntering = true;
#if WINDOWS_PHONE || ANDROID || WINDOWS_FORMS
                ((RelayCommand)this.Enter).RaiseCanExecuteChanged();
#endif
                this._gameClient = new Client(this.NickName);
                this._gameService.EnterAsync(this._gameClient, this._gameClient.Id);
            }
            catch (Exception)
            {
                this._hasEntered = false;
                this._isEntering = false;
#if WINDOWS_PHONE || ANDROID || WINDOWS_FORMS
                ((RelayCommand)this.Enter).RaiseCanExecuteChanged();
#endif
            }
        }

        private void OnChat()
        {
            if (!string.IsNullOrEmpty(this.ChatMessage))
            {
                this._gameService.SayAsync(this._gameClient, this.ChatMessage);
                this.ChatMessage = null;
            }
        }
        #endregion  // Commands

        #endregion  // Methods

        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(string propertyName)
        {
            if (this.PropertyChanged != null)
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        #region IMainViewModel
        public void AppendMessage(string text)
        {
            this.Messages += text + "\r\n";
        }

        public void ClientEntered()
        {
            this._hasEntered = true;
            this._isEntering = false;
#if WINDOWS_PHONE || ANDROID || WINDOWS_FORMS
            ((RelayCommand)this.Enter).RaiseCanExecuteChanged();
#else
            NotifyPropertyChanged("CanEnter");
#endif
        }
        #endregion  // IMainViewModel
    }
}
