﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GameServer.DataContracts;

namespace GameServer.Utils
{
    class Clients
    {
        private readonly Dictionary<Guid, ClientCallback> clients = new Dictionary<Guid, ClientCallback>();
        private static readonly object dictionaryLock = new object();
        private static Clients instance = null;
        private static readonly object instanceLock = new object();
        
        private Clients()
        {

        }

        public static Clients Instance
        {
            get
            {
                lock (instanceLock)
                {
                    if (instance == null)
                        instance = new Clients();

                    return instance;
                }
            }
        }

        public void AddClient(Client client, IGameServiceCallback callback)
        {
            lock (dictionaryLock)
            {
                if (clients.ContainsKey(client.Id))
                    throw new ArgumentException("Invalid client ID.", "client");

                clients.Add(client.Id, new ClientCallback(client, callback));
            }
        }

        public void RemoveClient(Client client)
        {
            lock (dictionaryLock)
            {
                clients.Remove(client.Id);
            }
        }

        public void NotifyAllThatClientEntered(Client clientEntered)
        {
            foreach (ClientCallback client in this.clients.Values)
                client.Callback.ClientEntered(clientEntered);
        }

        public void NotifyAllAboutChatMessage(Client clientTalking, string chatMessage)
        {
            foreach (ClientCallback client in this.clients.Values)
                client.Callback.ClientSaid(clientTalking, chatMessage);
        }

        private class ClientCallback
        {
            public Client Client { get; private set; }
            public IGameServiceCallback Callback { get; private set; }

            public ClientCallback(Client client, IGameServiceCallback callback)
            {
                this.Client = client;
                this.Callback = callback;
            }
        }
    }
}
