﻿using System;
using System.ComponentModel;
using System.Windows.Forms;
using System.Reflection;

namespace WindowsForms.Client.MVVM
{

	public class ViewModelBindingSource : BindingSource
	{
		private bool IsDesignMode { get; set; }

		public ViewModelBindingSource(IContainer container)
			: base(container)
		{
			this.IsDesignMode = LicenseManager.UsageMode == LicenseUsageMode.Designtime;
		}

		private object _viewModel;
		private Type _viewModelType;

		[Category("MVVM"), 
		DefaultValue((string)null), 
		AttributeProvider(typeof(IListSource))]
		public Type ViewModel
		{
			get { return _viewModelType; }
			set
			{

				SuspendBinding();
				_viewModelType = value;
				Type[] parameterTypes = { typeof(bool) };
				object[] parameters = { this.IsDesignMode };
				ConstructorInfo constructor = _viewModelType.GetConstructor(parameterTypes);
				_viewModel = constructor.Invoke(parameters);
				Add(_viewModel);
				MoveFirst();
				ResumeBinding();
			}
		}

		public override object this[int index]
		{
			get
			{
				return _viewModel;
			}
			set
			{
				base[index] = value;
			}
		}
	}
}
