﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Input;

namespace GameFx.WindowsForms.Client.MVVM
{

	[ProvideProperty("Command", typeof(Button))]
	public class CommandExtenderProvider : Component, IExtenderProvider
	{
		private readonly Dictionary<Button, string> _buttonsWithCommands;
		private readonly List<Button> _uninitializedButtons;

		public CommandExtenderProvider()
		{
			_buttonsWithCommands = new Dictionary<Button, string>();
			_uninitializedButtons = new List<Button>();
		}

		public bool CanExtend(object extendee)
		{
			return extendee is Button;
		}

		private BindingSource _viewModelSource;

		[Category("MVVM")]
		public BindingSource ViewModelSource
		{
			get { return _viewModelSource; }
			set
			{
				_viewModelSource = value;
				InitializeUninitializedButtons();
			}
		}

		private void InitializeUninitializedButtons()
		{
			foreach (var button in _uninitializedButtons)
			{
				var commandName = _buttonsWithCommands[button];
				SetCommand(button, commandName);
			}

			_uninitializedButtons.Clear();
		}


		[Editor(typeof(CommandEditor), typeof(UITypeEditor))]
		public string GetCommand(Button button)
		{
			if (!_buttonsWithCommands.ContainsKey(button))
			{
				return null;
			}
			var command = _buttonsWithCommands[button];
			return command;

		}

		public void SetCommand(Button button, string commandName)
		{
			_buttonsWithCommands[button] = commandName;

			if (null == ViewModelSource || null == ViewModelSource.Current)
			{
				if (!this._uninitializedButtons.Contains(button))
					this._uninitializedButtons.Add(button);
				
				return;
			}

			var property = ViewModelSource.Current.GetType().GetProperty(commandName);

			if (null != property)
			{
				var command = property.GetValue(ViewModelSource.Current, null) as ICommand;

				if (null != command)
				{
					button.Enabled = command.CanExecute(null);

					command.CanExecuteChanged += (s, e) => button.Enabled = command.CanExecute(null);
					button.Click += (s, e) =>
										{
											if (command.CanExecute(null))
											{
												command.Execute(null);
											}
										};
				}
			}
		}

	}
}
