﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GameFx.Entities.Messages
{
    public class ClientListMessage : MessageBase
    {
        public IEnumerable<Client> Clients { get; private set; }

        protected ClientListMessage()
            : base()
        {

        }

        public ClientListMessage(IEnumerable<Client> clients)
            : base()
        {
            this.Clients = clients;
        }

        #region IXmlSerializable
        public override void ReadXml(System.Xml.XmlReader reader)
        {
            // Clients
            reader.ReadToFollowing("Clients");
            List<Client> clients = new List<Client>();

            while (reader.Read())
            {
                // Client
                if (reader.IsStartElement("Client"))
                {
                    Client client = Client.CreateFromXml(reader);
                    clients.Add(client);
                }
            }

            this.Clients = clients.AsEnumerable();
        }

        public override void WriteXml(System.Xml.XmlWriter writer)
        {
            // Clients
            writer.WriteStartElement("Clients");

            foreach (Client client in this.Clients)
            {
                // Client
                writer.WriteStartElement("Client");
                client.WriteXml(writer);
                writer.WriteEndElement();
            }

            writer.WriteEndElement();
        }
        #endregion  // IXmlSerializable
    }
}
