﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SignalR.Hubs;
using GameFx.Entities;
using GameFx.Entities.Messages;

namespace GameFx.Server
{
    [HubName("Lobby")]
    public class LobbyHub : Hub
    {

        #region Properties
        public ILogger Logger { get; set; }
        public readonly Dictionary<Guid, Client> LobbyClients = new Dictionary<Guid, Client>();
        #endregion  // Properties

        #region Constructors
        public LobbyHub()
        {

        }

        public LobbyHub(ILogger logger)
            : this()
        {
            this.Logger = logger;
        }
        #endregion  // Constructors

        #region Methods
        public void Join(Client client)
        {
            if (this.LobbyClients.Where(x => x.Value.NickName.Equals(client.NickName)).Count().Equals(0))
            {
                this.LobbyClients.Add(client.Token.Id, client);
                BroadcastMessage(new ChatMessage(client.Token, string.Format("{0} entered the chat", client.NickName)));
                //this.GetClients();
                Caller.Client = client;
            }
            else
                throw new Exception("This login is already in use");
        }

        public void BroadcastMessage(ChatMessage chatMessage)
        {
            this.Clients.incomingChatMessage(chatMessage);
        }

        public void Disconnect(Client client)
        {
            this.BroadcastMessage(new ChatMessage(client.Token, string.Format("{0} disconnected.", client.NickName)));
            this.LobbyClients.Remove(client.Token.Id);
        }

        //public void GetClients()
        //{
        //    System.Web.Script.Serialization.JavaScriptSerializer oSerializer = new System.Web.Script.Serialization.JavaScriptSerializer();
        //    string sJSON = oSerializer.Serialize(Clients);            
        //    this.Clients.userList(sJSON);
        //}
        #endregion  // Methods

    }
}
