using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.ServiceModel;

namespace Lokeiv.Games.Ludo.Forms.InternetGame
{
    public partial class LudoRoom : Form
    {
        private ServiceCallback.LudoServiceCallback _ludoServiceCallback;
        private InstanceContext _instanceContext;
        private LudoService.ILudoService _ludoService;
        
        private LudoService.Client _client;
        private LudoService.GameRoom _gameRoom;
        private List<LudoTable> _tableFormsOpen = new List<LudoTable>();

        public List<LudoTable> TableFormsOpen
        {
            get { return _tableFormsOpen; }            
        }

        public LudoTables LudoTables
        {
            get
            {
                if (this.roomAndChatPanel.Panel1.Controls.Count == 0)
                {
                    LudoTables control = new LudoTables { Dock = DockStyle.Fill };
                    control.OpenTable += new EventHandler<TableEventArgs>(OpenTable);
                    this.roomAndChatPanel.Panel1.Controls.Add(control);
                }

                return this.roomAndChatPanel.Panel1.Controls[0] as LudoTables;
            }
        }

        void OpenTable(object sender, TableEventArgs e)
        {
            foreach (LudoService.Table table in this._gameRoom.Tables)
            {
                if (table.Name == e.TableName)
                {
                    LudoTable ludoTable = new LudoTable(this, table, this._client, this._ludoService, this._ludoServiceCallback);
                    this._tableFormsOpen.Add(ludoTable);
                    ludoTable.Show();
                    break;
                }
            }
        }

        public LudoRoom()
        {
            InitializeComponent();

            // Create an instance of the ludo service callback handler
            this._ludoServiceCallback = new ServiceCallback.LudoServiceCallback();
            this._ludoServiceCallback.ClientEnteredGameRoom += new EventHandler<ServiceCallback.LudoServiceClientEventArgs>(ClientEnteredOrLeft);
            this._ludoServiceCallback.ClientLeftGameRoom += new EventHandler<ServiceCallback.LudoServiceClientEventArgs>(ClientEnteredOrLeft);
            this._ludoServiceCallback.ClientEnteredTable += new EventHandler<ServiceCallback.LudoServiceTableEventArgs>(TableChanges);
            this._ludoServiceCallback.ClientLeftTable += new EventHandler<ServiceCallback.LudoServiceTableEventArgs>(TableChanges);
            this._ludoServiceCallback.ClientSatDown += new EventHandler<ServiceCallback.LudoServiceTableEventArgs>(TableChanges);
            this._ludoServiceCallback.ClientRaisedUp += new EventHandler<ServiceCallback.LudoServiceTableEventArgs>(TableChanges);
            this._ludoServiceCallback.TableCreated += new EventHandler<ServiceCallback.LudoServiceTableEventArgs>(TableChanges);
            this._ludoServiceCallback.ClientSentMessageToGameRoom += new EventHandler<ServiceCallback.LudoServiceMessageEventArgs>(IncomingMessage);
            
            // Create a reference to the ludo service
            this._instanceContext = new InstanceContext(this._ludoServiceCallback);
            this._ludoService = new LudoService.LudoServiceClient(this._instanceContext);

            NickNameForm nickNameForm = new NickNameForm(this._ludoService);

            if (nickNameForm.ShowDialog() == DialogResult.OK)
            {
                this._client = nickNameForm.Client;
            }
            else
            {
                this.Close();
            }
        }

        void IncomingMessage(object sender, ServiceCallback.LudoServiceMessageEventArgs e)
        {
            this.chatHistoryTextBox.AppendText(string.Format("{0} says: {1}\n", e.NickName, e.Message));
        }

        void TableChanges(object sender, ServiceCallback.LudoServiceTableEventArgs e)
        {
            this.UpdateTablesList();
        }

        void ClientEnteredOrLeft(object sender, ServiceCallback.LudoServiceClientEventArgs e)
        {
            this.UpdateClientsList();            
        }


        /// <summary>
        /// Updates the list of clients.
        /// </summary>
        private void UpdateClientsList()
        {
            this.clientsListView.Items.Clear();
            this._gameRoom = this._ludoService.GetGameRoom();

            foreach(LudoService.Client client in this._gameRoom.Clients)
            {
                this.clientsListView.Items.Add(client.NickName);
            }
        }

        /// <summary>
        /// Updates the list of tables.
        /// </summary>
        private void UpdateTablesList()
        {
            this.LudoTables.Tables.Clear();
            this.LudoTables.Tables.AddRange(this._gameRoom.Tables);
            this.LudoTables.UpdateView();
        }

        private void LudoRoom_Shown(object sender, EventArgs e)
        {
            // Load list of clients and room state once the form is displayed
            this._gameRoom = this._ludoService.GetGameRoom();
            this.UpdateClientsList();
            this.UpdateTablesList();
        }

        private void LudoRoom_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (this._tableFormsOpen.Count == 0)
            {
                Application.Exit();
            }
        }

        private void chatTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter || e.KeyCode == Keys.Return)
            {
                this._ludoService.SendMessageToGameRoom(this._client.NickName, this.chatTextBox.Text);
                this.chatTextBox.Text = string.Empty;
            }
        }
    }
}