using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Lokeiv.Games.Ludo.Forms.InternetGame.LudoService;

namespace Lokeiv.Games.Ludo.Forms.InternetGame
{
    public partial class LudoTables : UserControl
    {
        public event EventHandler<TableEventArgs> OpenTable;

        private List<Table> _tables = new List<Table>();

        public List<Table> Tables
        {
            get { return this._tables; }
            set { this._tables = value; this.UpdateView(); }
        }

        public LudoTables()
        {
            InitializeComponent();
        }

        public void UpdateView()
        {
            this.tablesListView.BeginUpdate();

            try
            {
                this.tablesListView.Items.Clear();

                foreach (Table table in this.Tables)
                {
                    ListViewItem tableName = new ListViewItem(table.Name);
                    tableName.SubItems.Add(new ListViewItem.ListViewSubItem(tableName, string.Empty));
                    tableName.SubItems.Add(new ListViewItem.ListViewSubItem(tableName, string.Empty));
                    tableName.SubItems.Add(new ListViewItem.ListViewSubItem(tableName, string.Empty));
                    tableName.SubItems.Add(new ListViewItem.ListViewSubItem(tableName, string.Empty));

                    string operatorName = string.Empty;
                    if (table.Operator != null) operatorName = table.Operator.NickName;

                    if (table.RedPlayer != null)
                    {
                        if (table.RedPlayer.NickName == operatorName)
                            tableName.SubItems[0].Text = operatorName + " *";
                        else
                            tableName.SubItems[0].Text = table.RedPlayer.NickName;
                    }

                    if (table.YellowPlayer != null)
                    {
                        if (table.YellowPlayer.NickName == operatorName)
                            tableName.SubItems[0].Text = operatorName + " *";
                        else
                            tableName.SubItems[0].Text = table.YellowPlayer.NickName;
                    }

                    if (table.GreenPlayer != null)
                    {
                        if (table.GreenPlayer.NickName == operatorName)
                            tableName.SubItems[0].Text = operatorName + " *";
                        else
                            tableName.SubItems[0].Text = table.GreenPlayer.NickName;
                    }

                    if (table.BluePlayer != null)
                    {
                        if (table.BluePlayer.NickName == operatorName)
                            tableName.SubItems[0].Text = operatorName + " *";
                        else
                            tableName.SubItems[0].Text = table.BluePlayer.NickName;
                    }

                    this.tablesListView.Items.Add(tableName);
                }
            }
            catch
            {
                throw;
            }
            finally
            {
                this.tablesListView.EndUpdate();
            }
        }

        private void tablesListView_DoubleClick(object sender, EventArgs e)
        {
            ListViewItem focused = this.tablesListView.FocusedItem;
            this.OnOpenTable(new TableEventArgs(focused.Text));            
        }

        protected void OnOpenTable(TableEventArgs e)
        {
            if (this.OpenTable != null)
                OpenTable(this, e);
        }
    }
}
