using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Web.Services.Protocols;
using Lokeiv.Games.Ludo.Resources;

namespace Lokeiv.Games.Ludo.Forms.InternetGame
{
    public partial class NickNameForm : Form
    {
        private LudoService.ILudoService _ludoService;
        private LudoService.Client _client;

        public LudoService.Client Client
        {
            get { return _client; }            
        }
	

        public NickNameForm(LudoService.ILudoService ludoService)
        {
            InitializeComponent();
            this._ludoService = ludoService;
        }

        /// <summary>
        /// Validates the given nick name before the server is called 
        /// to add the client.
        /// </summary>
        /// <returns>True if validation succeeds</returns>
        private bool LocalValidation()
        {
            if (this.nickNameTextBox.Text == string.Empty)
            {
                this.errorProvider.SetError(this.nickNameTextBox, Errors.NickNameMissing);
                return false;
            }

            this.errorProvider.SetError(this.nickNameTextBox, string.Empty);
            return true;
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            if (this.LocalValidation())
            {
                try
                {
                    this._client = this._ludoService.EnterWithNickName(this.nickNameTextBox.Text);
                    this.errorProvider.SetError(this.nickNameTextBox, string.Empty);
                }
                catch(SoapException se)
                {
                    this.errorProvider.SetError(this.nickNameTextBox, se.Message);                    
                }
            }
        }

        private void NickNameForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (this.errorProvider.GetError(this.nickNameTextBox) != string.Empty)
            {
                e.Cancel = true;
            }
        }
    }
}