using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;

namespace Lokeiv.Games.Ludo.Model
{
    public class MoveablePiece : IXmlSerializable
    {
        private Piece _piece = null;
        private Square _square = null;

        #region Properties
        public Piece Piece
        {
            get { return this._piece; }
        }

        public Square Square
        {
            get { return this._square; }
        }
        #endregion

        #region Contructors
        public MoveablePiece(Piece piece, Square square)
        {
            this._piece = piece;
            this._square = square;
        }
        #endregion

        #region IXmlSerializable
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(System.Xml.XmlReader reader)
        {
            throw new NotImplementedException();
        }

        public static MoveablePiece ReadXml(System.Xml.XmlReader reader, Board board)
        {
            Piece piece = null;
            Square square = null;

            while (reader.Read())
            {
                if (reader.NodeType == System.Xml.XmlNodeType.Element)
                {
                    switch (reader.Name.ToLower())
                    {
                        case "piece":
                            piece = Piece.ReadXml(reader.ReadSubtree(), board);
                            reader.Skip();
                            break;
                        case "square":
                            square = Lokeiv.Games.Ludo.Model.Square.ReadXmlReference(reader.ReadSubtree(), board);
                            reader.Skip();
                            break;
                    }
                }
            }

            if (piece != null && square != null)
                return new MoveablePiece(piece, square);

            return null;
        }

        public static List<MoveablePiece> ReadCollectionFromXml(System.Xml.XmlReader reader, Board board)
        {
            List<MoveablePiece> movablePieces = new List<MoveablePiece>();

            while (reader.Read())
            {
                if (reader.NodeType == System.Xml.XmlNodeType.Element)
                {
                    switch(reader.Name.ToLower())
                    {
                        case "movablepiece":
                            movablePieces.Add(MoveablePiece.ReadXml(reader.ReadSubtree(), board));
                            reader.Skip();
                            break;
                    }
                }
            }

            return movablePieces;
        }

        public void WriteXml(System.Xml.XmlWriter writer)
        {
            // Write Piece
            writer.WriteStartElement("Piece");
            if (this.Piece != null) this.Piece.WriteXmlReference(writer);
            writer.WriteEndElement();

            // Write Square
            writer.WriteStartElement("Square");
            if (this.Square != null) this.Square.WriteXmlReference(writer);
            writer.WriteEndElement();
        }
        #endregion
    }
}
