﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Ludo.WP7.Client.ViewModels;
using Ludo.WP7.Client.Views;

namespace Ludo.WP7.Client
{
    public partial class MainPage : BasePage
    {
        public ViewModels.MainViewModel ViewModel
        {
            get { return this.DataContext as ViewModels.MainViewModel; }
        }

        // Constructor
        public MainPage()
        {
            InitializeComponent();

            // Set the data context of the listbox control to the sample data
            DataContext = App.ViewModel;
            this.Loaded += new RoutedEventHandler(MainPage_Loaded);
        }

        // Load data for the ViewModel Items
        private void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            var viewModel = this.DataContext as MainViewModel;
            viewModel.NavigateToNewGame -= NavigateToNewGame;  // Avoid duplicate event subscriptions
            viewModel.NavigateToNewGame += NavigateToNewGame;
            viewModel.ViewLoaded();            
        }


        protected override void OnNavigatedFrom(System.Windows.Navigation.NavigationEventArgs e)
        {
            base.OnNavigatedFrom(e);
            this.ViewModel.IsRunning = false;
        }

        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            this.ViewModel.IsRunning = true;
        }

        private void boardCanvas_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            Point point = e.GetPosition(boardCanvas);
            var viewModel = this.DataContext as MainViewModel;
            viewModel.HandleMouseClickInBoard(point);
        }

        private void NewGameButton_Click(object sender, EventArgs e)
        {
           if(this.ViewModel.NewGame.CanExecute(null))
               this.ViewModel.NewGame.Execute(null);            
        }

        private void QuitButton_Click(object sender, EventArgs e)
        {
            if (this.ViewModel.Exit.CanExecute(null))
                this.ViewModel.Exit.Execute(null);
        }

        private void NavigateToNewGame(object sender, EventArgs e)
        {
            NavigationService.Navigate(new Uri(@"/Views/NewGamePortraitPage.xaml", UriKind.Relative));
        }

        private void ApplicationBarMenuItem_Click(object sender, EventArgs e)
        {
            NavigationService.Navigate(new Uri(@"/Views/AboutPage.xaml", UriKind.Relative));
        }

        private void MainPage_OrientationChanged(object sender, OrientationChangedEventArgs e)
        {
            if ((e.Orientation & PageOrientation.Portrait) == PageOrientation.Portrait)
            {
                // Show application title
                this.ApplicationTitle.Visibility = System.Windows.Visibility.Collapsed;

                this.controlsLandscapeGrid.Visibility = System.Windows.Visibility.Collapsed;
            }
            else
            {
                // Not enough space for the title, so hide it
                this.ApplicationTitle.Visibility = System.Windows.Visibility.Collapsed;

                // Put controls in a column to the right of the board
                this.controlsPortraitGrid.Visibility = System.Windows.Visibility.Collapsed;                
                this.controlsPortraitGrid.Children.Remove(this.redPlayer);
                this.controlsPortraitGrid.Children.Remove(this.yellowPlayer);
                this.controlsPortraitGrid.Children.Remove(this.greenPlayer);
                this.controlsPortraitGrid.Children.Remove(this.bluePlayer);
                this.controlsPortraitGrid.Children.Remove(this.theDie);

                this.controlsLandscapeGrid.Children.Add(this.redPlayer);
                this.redPlayer.SetValue(Grid.ColumnProperty, 0);
                this.controlsLandscapeGrid.Children.Add(this.yellowPlayer);
                this.yellowPlayer.SetValue(Grid.ColumnProperty, 1);
                this.controlsLandscapeGrid.Children.Add(this.theDie);
                this.theDie.SetValue(Grid.ColumnProperty, 2);
                this.controlsLandscapeGrid.Children.Add(this.greenPlayer);
                this.greenPlayer.SetValue(Grid.ColumnProperty, 3);
                this.controlsLandscapeGrid.Children.Add(this.bluePlayer);
                this.bluePlayer.SetValue(Grid.ColumnProperty, 4);
                this.controlsLandscapeGrid.Visibility = System.Windows.Visibility.Visible;
            }
        }
    }
}