﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;
using Ludo.WP7.Client.Views;

namespace Ludo.WP7.Client.Utils
{

    /// <summary>
    /// A class to help navigating from new game to main page while keeping a perfect navigation history.
    /// This class will no longer be needed when dropping 7.0 of WP OS, as the 7.1 version provides a 
    /// method to take pages out of the back history when necessary.
    /// </summary>
    /// <remarks>This class is not able to function without BasePage, so as long as this class is needed, 
    /// all pages must inherit BasePage.</remarks>
    public sealed class NavigationStack
    {
        #region Properties
        private Stack<Type> _navigationStack;
        public Type NavigatingFrom { get; set; }
        public Type Intention { get; set; }
        #endregion  // Properties

        #region Singleton pattern
        private static readonly object _instanceLock = new object();
        private static NavigationStack _instance;        

        public static NavigationStack Instance 
        { 
            get 
            {
                lock (_instanceLock)
                {
                    if (_instance == null)
                        _instance = new NavigationStack();

                    return _instance;
                }
            }
        }

        private NavigationStack()
        {
            this._navigationStack = new Stack<Type>();
        }
        #endregion  // Singleton pattern

        public void Push(Type pageType)
        {
            this._navigationStack.Push(pageType);
        }

        public Type Pop()
        {
            if (this._navigationStack.Count == 0)
                return null;

            return this._navigationStack.Pop();
        }

        public Type Peek()
        {
            if (this._navigationStack.Count == 0)
                return null;

            return this._navigationStack.Peek();
        }

        public bool IsNavigatingFrom(Type pageType)
        {
            return this.NavigatingFrom == pageType;
        }
    }
}
