﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using Ludo.WP7.Client.Utils;
using Lokeiv.Games.Ludo;
using System.Windows.Threading;
using System.Xml.Serialization;

namespace Ludo.WP7.Client.ViewModels
{
    public class DieViewModel : INotifyPropertyChanged, IXmlSerializable
    {

        #region Properties
        private Dispatcher _dispatcher;
        [XmlIgnore]
        public ICommand RollDie { get; private set; }    

        private Engine _ludoEngine;
        [XmlIgnore]
        public Engine LudoEngine
        {
            get { return _ludoEngine; }
            set
            {
                if (_ludoEngine != value)
                {
                    if (_ludoEngine != null)
                        _ludoEngine.DieRolled -= DieRolled;

                    _ludoEngine = value;

                    if (_ludoEngine != null)
                        _ludoEngine.DieRolled += DieRolled;

                    NotifyPropertyChanged("LudoEngine");
                }
            }
        }
        
        private int _dieValue = 6;
        public int DieValue
        {
            get { return _dieValue; }
            set 
            {
                if (_dieValue != value)
                {
                    _dieValue = value;
                    NotifyPropertyChanged("DieValue");
                    NotifyPropertyChanged("DieImageSource");
                }
            }
        }

        public string DieImageSource
        {
            get
            {
                switch (DieValue)
                {
                    case 1:
                        return @"\Images\Die_1.png";                        
                    case 2:
                        return @"\Images\Die_2.png";                        
                    case 3:
                        return @"\Images\Die_3.png";                        
                    case 4:
                        return @"\Images\Die_4.png";                        
                    case 5:
                        return @"\Images\Die_5.png";                        
                    default:
                        return @"\Images\Die_6.png";                        
                }
            }
        }

        #endregion

        #region Constructors

        public DieViewModel(Engine ludoEngine) 
        {
            _dispatcher = Deployment.Current.Dispatcher;
            LudoEngine = ludoEngine;
            RollDie = new RelayCommand(OnRollDie);
        }

        #endregion

        #region Methods

        #region Event handlers

        private void DieRolled(object sender, ActionEventArgs e)
        {
            if (_dispatcher.CheckAccess())
            {
                this.DieValue = (int)LudoEngine.Die.Value.Value;
            }
            else
            {
                _dispatcher.BeginInvoke(() =>
                {
                    this.DieValue = (int)LudoEngine.Die.Value.Value;
                });        
            }
        }

        #endregion

        #region Commands

        /// <summary>
        /// Rolls the die when the user interacts with the die in the view.
        /// </summary>
        private void OnRollDie()
        {
            //throw new Exception("Test exception");
            if (LudoEngine != null)
                LudoEngine.RollDie();
        }

        #endregion  // Commands

        #endregion  // Methods

        #region INotifyPropertyChanged
        /// <summary>
        /// Event to notify the view when a property has changed.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Fires the <see cref="PropertyChanged"/> event for the given <paramref name="propertyName"/>.
        /// </summary>
        /// <param name="propertyName">Name of property that has changed.</param>
        private void NotifyPropertyChanged(String propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion  // INotifyPropertyChanged

        #region IXmlSerializable
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(System.Xml.XmlReader reader)
        {
            
        }

        public void WriteXml(System.Xml.XmlWriter writer)
        {
            writer.WriteAttributeString("Value", this.DieValue.ToString());
        }
        #endregion
    }
}
