﻿using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Threading;
using System.Windows.Navigation;
using Lokeiv.Games.Dice;
using Lokeiv.Games.Ludo.Model;
using Lokeiv.Games.Ludo;
using Ludo.WP7.Model;
using Ludo.WP7.Client.Utils;
using Ludo.WP7.Client.Utils.BoardCoordinates;
using System.Xml.Serialization;

namespace Ludo.WP7.Client.ViewModels
{
    public class MainViewModel : INotifyPropertyChanged, IXmlSerializable
    {

        #region Events
        
        /// <summary>
        /// Notifies the view to navigate to the new game page.
        /// </summary>        
        [XmlIgnore]
        public EventHandler<EventArgs> NavigateToNewGame;

        #endregion  // Events

        #region Properties
        
        [XmlIgnore]
        private Dispatcher _dispatcher;
        [XmlElement]
        private Engine _engine;
        private BoardCoordinates _boardCoordinates;
        private List<MoveablePiece> _validMoves;

        private Utils.BoardCoordinates.Views _view;
        public Utils.BoardCoordinates.Views View 
        {
            get { return _view; }
            set
            {
                if (_view != value)
                {
                    _view = value;
                    this.UpdateBoardCoordinates();
                }
            }
        }

        public PlayerViewModelCollection Players { get; set; }
        [XmlIgnore]
        public ICommand NewGame { get; private set; }
        [XmlIgnore]
        public ICommand Exit { get; private set; }
        [XmlIgnore]
        public ICommand StartNewGame { get; private set; }                 
                
        private DieViewModel _die;
        public DieViewModel Die
        {
            get { return _die; }
            set
            {
                if (_die != value)
                {
                    _die = value;
                    NotifyPropertyChanged("Die");
                }
            }
        }
        
        private NewGameViewModel _newGameViewModel;
        public NewGameViewModel NewGameViewModel
        {
            get { return _newGameViewModel; }
            set
            {
                if (_newGameViewModel != value)
                {
                    _newGameViewModel = value;
                    NotifyPropertyChanged("NewGameViewModel");
                }
            }
        }

        private SettingsViewModel _settingsViewModel;
        public SettingsViewModel SettingsViewModel
        {
            get { return _settingsViewModel; }
            set
            {
                if (_settingsViewModel != value)
                {
                    _settingsViewModel = value;
                    NotifyPropertyChanged("SettingsViewModel");
                }
            }
        }

        private ErrorViewModel _errorViewModel;
        public ErrorViewModel ErrorViewModel
        {
            get { return _errorViewModel; }
            set
            {
                if (this._errorViewModel != value)
                {
                    this._errorViewModel = value;
                    NotifyPropertyChanged("ErrorViewModel");
                }
            }
        }
        
        private string _actionText;
        public string ActionText
        {
            get { return _actionText; }
            private set
            {
                if (_actionText != value)
                {
                    _actionText = value;
                    NotifyPropertyChanged("ActionText");
                }
            }
        }

        public bool IsRunning
        {
            get { return this._engine.IsRunning; }
            set 
            {
                if (value == true)
                    this._engine.StartGame();
                else
                    this._engine.EndGame();
            }
        }

        #region Pieces view models
        
        private PieceViewModel _redPiece1;
        public PieceViewModel RedPiece1 
        {
            get { return _redPiece1; }
            set
            {
                if (_redPiece1 != value)
                {
                    _redPiece1 = value;
                    NotifyPropertyChanged("RedPiece1");
                }
            }
        }

        private PieceViewModel _redPiece2;
        public PieceViewModel RedPiece2
        {
            get { return _redPiece2; }
            set
            {
                if (_redPiece2 != value)
                {
                    _redPiece2 = value;
                    NotifyPropertyChanged("RedPiece2");
                }
            }
        }

        private PieceViewModel _redPiece3;
        public PieceViewModel RedPiece3
        {
            get { return _redPiece3; }
            set
            {
                if (_redPiece3 != value)
                {
                    _redPiece3 = value;
                    NotifyPropertyChanged("RedPiece3");
                }
            }
        }

        private PieceViewModel _redPiece4;
        public PieceViewModel RedPiece4
        {
            get { return _redPiece4; }
            set
            {
                if (_redPiece4 != value)
                {
                    _redPiece4 = value;
                    NotifyPropertyChanged("RedPiece4");
                }
            }
        }

        private PieceViewModel _yellowPiece1;
        public PieceViewModel YellowPiece1
        {
            get { return _yellowPiece1; }
            set
            {
                if (_yellowPiece1 != value)
                {
                    _yellowPiece1 = value;
                    NotifyPropertyChanged("YellowPiece1");
                }
            }
        }

        private PieceViewModel _yellowPiece2;
        public PieceViewModel YellowPiece2
        {
            get { return _yellowPiece2; }
            set
            {
                if (_yellowPiece2 != value)
                {
                    _yellowPiece2 = value;
                    NotifyPropertyChanged("YellowPiece2");
                }
            }
        }

        private PieceViewModel _yellowPiece3;
        public PieceViewModel YellowPiece3
        {
            get { return _yellowPiece3; }
            set
            {
                if (_yellowPiece3 != value)
                {
                    _yellowPiece3 = value;
                    NotifyPropertyChanged("YellowPiece3");
                }
            }
        }

        private PieceViewModel _yellowPiece4;
        public PieceViewModel YellowPiece4
        {
            get { return _yellowPiece4; }
            set
            {
                if (_yellowPiece4 != value)
                {
                    _yellowPiece4 = value;
                    NotifyPropertyChanged("YellowPiece4");
                }
            }
        }

        private PieceViewModel _greenPiece1;
        public PieceViewModel GreenPiece1
        {
            get { return _greenPiece1; }
            set
            {
                if (_greenPiece1 != value)
                {
                    _greenPiece1 = value;
                    NotifyPropertyChanged("GreenPiece1");
                }
            }
        }

        private PieceViewModel _greenPiece2;
        public PieceViewModel GreenPiece2
        {
            get { return _greenPiece2; }
            set
            {
                if (_greenPiece2 != value)
                {
                    _greenPiece2 = value;
                    NotifyPropertyChanged("GreenPiece2");
                }
            }
        }

        private PieceViewModel _greenPiece3;
        public PieceViewModel GreenPiece3
        {
            get { return _greenPiece3; }
            set
            {
                if (_greenPiece3 != value)
                {
                    _greenPiece3 = value;
                    NotifyPropertyChanged("GreenPiece3");
                }
            }
        }

        private PieceViewModel _greenPiece4;
        public PieceViewModel GreenPiece4
        {
            get { return _greenPiece4; }
            set
            {
                if (_greenPiece4 != value)
                {
                    _greenPiece4 = value;
                    NotifyPropertyChanged("GreenPiece4");
                }
            }
        }

        private PieceViewModel _bluePiece1;
        public PieceViewModel BluePiece1
        {
            get { return _bluePiece1; }
            set
            {
                if (_bluePiece1 != value)
                {
                    _bluePiece1 = value;
                    NotifyPropertyChanged("BluePiece1");
                }
            }
        }

        private PieceViewModel _bluePiece2;
        public PieceViewModel BluePiece2
        {
            get { return _bluePiece2; }
            set
            {
                if (_bluePiece2 != value)
                {
                    _bluePiece2 = value;
                    NotifyPropertyChanged("BluePiece2");
                }
            }
        }

        private PieceViewModel _bluePiece3;
        public PieceViewModel BluePiece3
        {
            get { return _bluePiece3; }
            set
            {
                if (_bluePiece3 != value)
                {
                    _bluePiece3 = value;
                    NotifyPropertyChanged("BluePiece3");
                }
            }
        }

        private PieceViewModel _bluePiece4;
        public PieceViewModel BluePiece4
        {
            get { return _bluePiece4; }
            set
            {
                if (_bluePiece4 != value)
                {
                    _bluePiece4 = value;
                    NotifyPropertyChanged("BluePiece4");
                }
            }
        }

        #endregion  // Pieces color, position and visibility properties

        #region Player view models

        private PlayerViewModel _redPlayer;
        public PlayerViewModel RedPlayer 
        { 
            get
            {
                return _redPlayer;
            }
            private set
            {
                if (_redPlayer != value)
                {
                    _redPlayer = value;
                    NotifyPropertyChanged("RedPlayer");
                }
            }
        }

        private PlayerViewModel _yellowPlayer;
        public PlayerViewModel YellowPlayer 
        { 
            get
            {
                return _yellowPlayer;
            }
            private set
            {
                if (_yellowPlayer != value)
                {
                    _yellowPlayer = value;
                    NotifyPropertyChanged("YellowPlayer");
                }
            }
        }

        private PlayerViewModel _greenPlayer;
        public PlayerViewModel GreenPlayer 
        { 
            get
            {
                return _greenPlayer;
            }
            private set
            {
                if (_greenPlayer != value)
                {
                    _greenPlayer = value;
                    NotifyPropertyChanged("GreenPlayer");
                }
            }
        }

        private PlayerViewModel _bluePlayer;
        public PlayerViewModel BluePlayer 
        { 
            get
            {
                return _bluePlayer;
            }
            private set
            {
                if (_bluePlayer != value)
                {
                    _bluePlayer = value;
                    NotifyPropertyChanged("BluePlayer");
                }
            }
        }

        #endregion  // Player action properties

        #endregion  // Properties

        #region Constructors

        /// <summary>
        /// Constructor.
        /// </summary>
        public MainViewModel()
        {
            _boardCoordinates = BoardCoordinates.CreateBoardCoordinates(Utils.BoardCoordinates.Views.WindowsPhone7Portrait);            
            _dispatcher = Deployment.Current.Dispatcher;
            this.Die = new DieViewModel(this._engine);
            this.RedPiece1 = new PieceViewModel(Lokeiv.Games.Ludo.Model.Color.Red);
            this.RedPiece2 = new PieceViewModel(Lokeiv.Games.Ludo.Model.Color.Red);
            this.RedPiece3 = new PieceViewModel(Lokeiv.Games.Ludo.Model.Color.Red);
            this.RedPiece4 = new PieceViewModel(Lokeiv.Games.Ludo.Model.Color.Red);
            this.YellowPiece1 = new PieceViewModel(Lokeiv.Games.Ludo.Model.Color.Yellow);
            this.YellowPiece2 = new PieceViewModel(Lokeiv.Games.Ludo.Model.Color.Yellow);
            this.YellowPiece3 = new PieceViewModel(Lokeiv.Games.Ludo.Model.Color.Yellow);
            this.YellowPiece4 = new PieceViewModel(Lokeiv.Games.Ludo.Model.Color.Yellow);
            this.GreenPiece1 = new PieceViewModel(Lokeiv.Games.Ludo.Model.Color.Green);
            this.GreenPiece2 = new PieceViewModel(Lokeiv.Games.Ludo.Model.Color.Green);
            this.GreenPiece3 = new PieceViewModel(Lokeiv.Games.Ludo.Model.Color.Green);
            this.GreenPiece4 = new PieceViewModel(Lokeiv.Games.Ludo.Model.Color.Green);
            this.BluePiece1 = new PieceViewModel(Lokeiv.Games.Ludo.Model.Color.Blue);
            this.BluePiece2 = new PieceViewModel(Lokeiv.Games.Ludo.Model.Color.Blue);
            this.BluePiece3 = new PieceViewModel(Lokeiv.Games.Ludo.Model.Color.Blue);
            this.BluePiece4 = new PieceViewModel(Lokeiv.Games.Ludo.Model.Color.Blue);
            this.RedPlayer = new PlayerViewModel(this, new Player("Red player", PlayerType.Human, Lokeiv.Games.Ludo.Model.Color.Red));
            this.YellowPlayer = new PlayerViewModel(this, new Player("Yellow player", PlayerType.Computer, Lokeiv.Games.Ludo.Model.Color.Yellow));
            this.GreenPlayer = new PlayerViewModel(this, new Player("Green player", PlayerType.Computer, Lokeiv.Games.Ludo.Model.Color.Green));
            this.BluePlayer = new PlayerViewModel(this, new Player("Blue player", PlayerType.Computer, Lokeiv.Games.Ludo.Model.Color.Blue));
            this.NewGameViewModel = new ViewModels.NewGameViewModel(this, this.RedPlayer, this.YellowPlayer, this.GreenPlayer, this.BluePlayer);
            this.ErrorViewModel = new ViewModels.ErrorViewModel(this);
            NewGame = new RelayCommand(OnNewGame);
            Exit = new RelayCommand(OnExit);
            StartNewGame = new RelayCommand(OnStartNewGame);
            OnStartNewGame();
        }

        #endregion  // Constructors

        #region Methods

        #region Initialization

        /// <summary>
        /// Initializes the board content and starts the game engine.
        /// </summary>
        public void ViewLoaded()
        {
            this.UpdateBoardCoordinates();
            UpdatePiecePositions();
            _engine.StartGame();
        }  

        /// <summary>
        /// Creates players to the game.        
        /// </summary>
        private void InitializePlayers(bool deserialize)
        {
            if (deserialize)
                this.InitializePlayersFromDeserializedGame();
            else
                this.InitializePlayersFromNewGameViewModel();

            UpdatePieceVisibility();
        }

        /// <summary>
        /// Uses the <see cref="NewGameViewModel"/> to initialize players to a new game.
        /// </summary>
        private void InitializePlayersFromNewGameViewModel()
        {
            Players = new PlayerViewModelCollection();
            RedPlayer = new PlayerViewModel(this, NewGameViewModel.RedPlayer.GetPlayer());
            YellowPlayer = new PlayerViewModel(this, NewGameViewModel.YellowPlayer.GetPlayer());
            GreenPlayer = new PlayerViewModel(this, NewGameViewModel.GreenPlayer.GetPlayer());
            BluePlayer = new PlayerViewModel(this, NewGameViewModel.BluePlayer.GetPlayer());
            Players.Add(RedPlayer);

            if (NewGameViewModel.YellowPlayer.PlayerType != NewGamePlayerViewModel.NewGamePlayerType.NoPlayer)
                Players.Add(YellowPlayer);

            if (NewGameViewModel.GreenPlayer.PlayerType != NewGamePlayerViewModel.NewGamePlayerType.NoPlayer)
                Players.Add(GreenPlayer);

            if (NewGameViewModel.BluePlayer.PlayerType != NewGamePlayerViewModel.NewGamePlayerType.NoPlayer)
                Players.Add(BluePlayer);
        }

        /// <summary>
        /// Uses the deserialized ludo engine to update player view models from deserialized data.
        /// </summary>
        private void InitializePlayersFromDeserializedGame()
        {
            Players = new PlayerViewModelCollection();
            RedPlayer = new PlayerViewModel(this, this._engine.Players[Lokeiv.Games.Ludo.Model.Color.Red]);
            YellowPlayer = new PlayerViewModel(this, this._engine.Players[Lokeiv.Games.Ludo.Model.Color.Yellow]);
            GreenPlayer = new PlayerViewModel(this, this._engine.Players[Lokeiv.Games.Ludo.Model.Color.Green]);
            BluePlayer = new PlayerViewModel(this, this._engine.Players[Lokeiv.Games.Ludo.Model.Color.Blue]);
            
            if (this._engine.Players[Lokeiv.Games.Ludo.Model.Color.Red] != null)
                Players.Add(RedPlayer);

            if (this._engine.Players[Lokeiv.Games.Ludo.Model.Color.Yellow] != null)
                Players.Add(YellowPlayer);

            if (this._engine.Players[Lokeiv.Games.Ludo.Model.Color.Green] != null)
                Players.Add(GreenPlayer);

            if (this._engine.Players[Lokeiv.Games.Ludo.Model.Color.Blue] != null)
                Players.Add(BluePlayer);
        }

        /// <summary>
        /// Updates board coordinates for the current <see cref="View"/>.
        /// </summary>
        private void UpdateBoardCoordinates()
        {
            Utils.BoardCoordinates.Views view = this.View;            
            this._boardCoordinates = BoardCoordinates.CreateBoardCoordinates(view);
            this._boardCoordinates.CalculateBoardSize();
        }

        /// <summary>
        /// Updates positions of all pieces.
        /// </summary>
        private void UpdatePiecePositions()
        {
            RedPiece1.SetPiecePosition(GetPieceBounds(Lokeiv.Games.Ludo.Model.Color.Red, 0));
            RedPiece2.SetPiecePosition(GetPieceBounds(Lokeiv.Games.Ludo.Model.Color.Red, 1));
            RedPiece3.SetPiecePosition(GetPieceBounds(Lokeiv.Games.Ludo.Model.Color.Red, 2));
            RedPiece4.SetPiecePosition(GetPieceBounds(Lokeiv.Games.Ludo.Model.Color.Red, 3));
            YellowPiece1.SetPiecePosition(GetPieceBounds(Lokeiv.Games.Ludo.Model.Color.Yellow, 0));
            YellowPiece2.SetPiecePosition(GetPieceBounds(Lokeiv.Games.Ludo.Model.Color.Yellow, 1));
            YellowPiece3.SetPiecePosition(GetPieceBounds(Lokeiv.Games.Ludo.Model.Color.Yellow, 2));
            YellowPiece4.SetPiecePosition(GetPieceBounds(Lokeiv.Games.Ludo.Model.Color.Yellow, 3));
            GreenPiece1.SetPiecePosition(GetPieceBounds(Lokeiv.Games.Ludo.Model.Color.Green, 0));
            GreenPiece2.SetPiecePosition(GetPieceBounds(Lokeiv.Games.Ludo.Model.Color.Green, 1));
            GreenPiece3.SetPiecePosition(GetPieceBounds(Lokeiv.Games.Ludo.Model.Color.Green, 2));
            GreenPiece4.SetPiecePosition(GetPieceBounds(Lokeiv.Games.Ludo.Model.Color.Green, 3));
            BluePiece1.SetPiecePosition(GetPieceBounds(Lokeiv.Games.Ludo.Model.Color.Blue, 0));
            BluePiece2.SetPiecePosition(GetPieceBounds(Lokeiv.Games.Ludo.Model.Color.Blue, 1));
            BluePiece3.SetPiecePosition(GetPieceBounds(Lokeiv.Games.Ludo.Model.Color.Blue, 2));
            BluePiece4.SetPiecePosition(GetPieceBounds(Lokeiv.Games.Ludo.Model.Color.Blue, 3));
        }

        /// <summary>
        /// Updates labels of all pieces.
        /// </summary>
        private void UpdatePieceLabels()
        {
            RedPiece1.PieceCount = this.GetPieceCount(Lokeiv.Games.Ludo.Model.Color.Red, 0);
            RedPiece2.PieceCount = this.GetPieceCount(Lokeiv.Games.Ludo.Model.Color.Red, 1);
            RedPiece3.PieceCount = this.GetPieceCount(Lokeiv.Games.Ludo.Model.Color.Red, 2);
            RedPiece4.PieceCount = this.GetPieceCount(Lokeiv.Games.Ludo.Model.Color.Red, 3);
            YellowPiece1.PieceCount = this.GetPieceCount(Lokeiv.Games.Ludo.Model.Color.Yellow, 0);
            YellowPiece2.PieceCount = this.GetPieceCount(Lokeiv.Games.Ludo.Model.Color.Yellow, 1);
            YellowPiece3.PieceCount = this.GetPieceCount(Lokeiv.Games.Ludo.Model.Color.Yellow, 2);
            YellowPiece4.PieceCount = this.GetPieceCount(Lokeiv.Games.Ludo.Model.Color.Yellow, 3);
            GreenPiece1.PieceCount = this.GetPieceCount(Lokeiv.Games.Ludo.Model.Color.Green, 0);
            GreenPiece2.PieceCount = this.GetPieceCount(Lokeiv.Games.Ludo.Model.Color.Green, 1);
            GreenPiece3.PieceCount = this.GetPieceCount(Lokeiv.Games.Ludo.Model.Color.Green, 2);
            GreenPiece4.PieceCount = this.GetPieceCount(Lokeiv.Games.Ludo.Model.Color.Green, 3);
            BluePiece1.PieceCount = this.GetPieceCount(Lokeiv.Games.Ludo.Model.Color.Blue, 0);
            BluePiece2.PieceCount = this.GetPieceCount(Lokeiv.Games.Ludo.Model.Color.Blue, 1);
            BluePiece3.PieceCount = this.GetPieceCount(Lokeiv.Games.Ludo.Model.Color.Blue, 2);
            BluePiece4.PieceCount = this.GetPieceCount(Lokeiv.Games.Ludo.Model.Color.Blue, 3);
        }

        /// <summary>
        /// Updates the <see cref="PieceViewModel.IsPieceInGame"/> property of all pieces.
        /// </summary>
        private void UpdatePieceVisibility()
        {
            YellowPiece1.IsPieceInGame = (YellowPlayer.PlayerType.HasValue);
            YellowPiece2.IsPieceInGame = (YellowPlayer.PlayerType.HasValue);
            YellowPiece3.IsPieceInGame = (YellowPlayer.PlayerType.HasValue);
            YellowPiece4.IsPieceInGame = (YellowPlayer.PlayerType.HasValue);
            GreenPiece1.IsPieceInGame = (GreenPlayer.PlayerType.HasValue);
            GreenPiece2.IsPieceInGame = (GreenPlayer.PlayerType.HasValue);
            GreenPiece3.IsPieceInGame = (GreenPlayer.PlayerType.HasValue);
            GreenPiece4.IsPieceInGame = (GreenPlayer.PlayerType.HasValue);
            BluePiece1.IsPieceInGame = (BluePlayer.PlayerType.HasValue);
            BluePiece2.IsPieceInGame = (BluePlayer.PlayerType.HasValue);
            BluePiece3.IsPieceInGame = (BluePlayer.PlayerType.HasValue);
            BluePiece4.IsPieceInGame = (BluePlayer.PlayerType.HasValue);
        }

        #endregion  // Initialization

        #region Helper methods
        /// <summary>
        /// Looks through all <see cref="Player"/>s in the current game, to se if
        /// a player with the given <paramref name="color"/> is in the game.
        /// </summary>
        /// <param name="color"><see cref="Lokeiv.Games.Ludo.Model.Color"/> of the player.</param>
        /// <returns><see cref="bool"/></returns>
        private bool IsPlayerInGame(Lokeiv.Games.Ludo.Model.Color color)
        {
            foreach (Player player in _engine.Players)
                if (player.Color == color)
                    return true;

            return false;
        }

        /// <summary>
        /// Gets the text to be displayed on a piece if serveral pieces of the same color is on the same square.
        /// </summary>        
        /// <returns><see cref="string"/> to be displayed on pieces that's on the same square.</returns>
        private string GetPieceText(Lokeiv.Games.Ludo.Model.Color color, int pieceIndex)
        {
            var count = GetPieceCount(color, pieceIndex);
            return count < 2 ? string.Empty : count.ToString();
        }

        /// <summary>
        /// Gets number of pieces of the same color on same square.
        /// </summary>
        /// <param name="color"><see cref="Lokeiv.Games.Ludo.Model.Color"/> of the piece</param>
        /// <param name="pieceIndex">Which of the four pieces, 0 through 3.</param>
        /// <returns>Number of same pieces on the same square as the one in question.</returns>
        private int GetPieceCount(Lokeiv.Games.Ludo.Model.Color color, int pieceIndex)
        {
            if (color == Lokeiv.Games.Ludo.Model.Color.White)
                throw new ArgumentOutOfRangeException("color", "Cannot be white. Must represent a player.");

            if (pieceIndex < 0 || pieceIndex > 3)
                throw new ArgumentOutOfRangeException("pieceIndex", "Must be between 0 and 3.");

            var piece = _engine.Board.GetPieceOfPlayer(color, pieceIndex);

            if (piece == null) 
                return 0;

            return piece.NumberOfSamePiecesOnSameSquare();
        }

        /// <summary>
        /// Gets the bounds for a <see cref="Piece"/> for the current <see cref="Utils.BoardCoordinates.Views"/>.
        /// </summary>
        /// <param name="color"><see cref="Lokeiv.Games.Ludo.Model.Color"/> of the player.</param>
        /// <param name="pieceIndex">Piece number 0 through 3.</param>
        /// <returns><see cref="Rect"/> for the visual bounds of the <see cref="Piece"/>.</returns>
        private Rect GetPieceBounds(Lokeiv.Games.Ludo.Model.Color color, int pieceIndex)
        {
            if (color == Lokeiv.Games.Ludo.Model.Color.White)
                throw new ArgumentOutOfRangeException("color", "Cannot be white. Must represent a player.");

            if(pieceIndex < 0 || pieceIndex > 3)
                throw new ArgumentOutOfRangeException("pieceIndex", "Must be between 0 and 3.");

            // If player is not in the game, return the inhouse position for the piece
            if (!this.IsPlayerInGame(color))
            {
                // The piece won't be displayed so the position doesn't really matter. Just return the inhouse position.
                Square square = this._engine.Board.GetHouseSquare(color, pieceIndex);
                return this._boardCoordinates.SquareBounds[square.Number];
            }
            else
            {
                // Check the model in which square the piece is, and use the board coordinates to find its bounds.
                Square square = this._engine.Board.GetPieceOfPlayer(color, pieceIndex).Square;
                return this._boardCoordinates.SquareBounds[square.Number];
            }
        }

        /// <summary>
        /// Gets the <see cref="Piece"/>s of color <paramref name="color"/> which is located at <paramref name="point"/>.
        /// </summary>
        /// <param name="point"><see cref="Point"/> at which to look for <see cref="Piece"/>s.</param>
        /// <param name="color"><see cref="Lokeiv.Games.Ludo.Model.Color"/> of <see cref="Piece"/>s to get.</param>
        /// <returns><see cref="List<Piece>"/> at the given <paramref name="point"/>.</returns>
        private List<Piece> GetPiecesAtPosition(Point point, Lokeiv.Games.Ludo.Model.Color color)
        {
            List<Piece> piecesHit = new List<Piece>();

            Rect piece0 = this.GetPieceBounds(color, 0);
            if (piece0.Contains(point))
                piecesHit.Add(this._engine.Board.GetPieceOfPlayer(color, 0));

            Rect piece1 = this.GetPieceBounds(color, 1);
            if (piece1.Contains(point))
                piecesHit.Add(this._engine.Board.GetPieceOfPlayer(color, 1));

            Rect piece2 = this.GetPieceBounds(color, 2);
            if (piece2.Contains(point))
                piecesHit.Add(this._engine.Board.GetPieceOfPlayer(color, 2));

            Rect piece3 = this.GetPieceBounds(color, 3);
            if (piece3.Contains(point))
                piecesHit.Add(this._engine.Board.GetPieceOfPlayer(color, 3));


            return piecesHit;
        }

        /// <summary>
        /// Gets a list of <see cref="Player"/>s from the list of <see cref="PlayerViewModel"/>s.
        /// </summary>
        /// <returns><see cref="PlayerCollection"/></returns>
        private PlayerCollection GetPlayersFromPlayerViewModels()
        {
            var players = new PlayerCollection();

            foreach (PlayerViewModel viewModel in Players)
                players.Add(viewModel.Player);

            return players;
        }

        /// <summary>
        /// Updates the bounds of pices that have been moved.
        /// </summary>
        /// <param name="piecesMoved"><see cref="List<MovablePiece>"/> for pieces that have moved.</param>
        private void UpdatePieceBounds(List<MoveablePiece> piecesMoved)
        {
            foreach (MoveablePiece pieceMoved in piecesMoved)
            {
                Piece piece = pieceMoved.Piece;
                Rect pieceBounds = GetPieceBounds(piece.Color, piece.Number);
                int pieceNumber = piece.Number + 1;
                
                switch(piece.Color)
                {
                    case Lokeiv.Games.Ludo.Model.Color.Red:
                        if (pieceNumber == 1) RedPiece1.SetPiecePosition(pieceBounds);
                        if (pieceNumber == 2) RedPiece2.SetPiecePosition(pieceBounds);
                        if (pieceNumber == 3) RedPiece3.SetPiecePosition(pieceBounds);
                        if (pieceNumber == 4) RedPiece4.SetPiecePosition(pieceBounds);
                        break;
                    case Lokeiv.Games.Ludo.Model.Color.Yellow:
                        if (pieceNumber == 1) YellowPiece1.SetPiecePosition(pieceBounds);
                        if (pieceNumber == 2) YellowPiece2.SetPiecePosition(pieceBounds);
                        if (pieceNumber == 3) YellowPiece3.SetPiecePosition(pieceBounds);
                        if (pieceNumber == 4) YellowPiece4.SetPiecePosition(pieceBounds);
                        break;
                    case Lokeiv.Games.Ludo.Model.Color.Green:
                        if (pieceNumber == 1) GreenPiece1.SetPiecePosition(pieceBounds);
                        if (pieceNumber == 2) GreenPiece2.SetPiecePosition(pieceBounds);
                        if (pieceNumber == 3) GreenPiece3.SetPiecePosition(pieceBounds);
                        if (pieceNumber == 4) GreenPiece4.SetPiecePosition(pieceBounds);
                        break;
                    case Lokeiv.Games.Ludo.Model.Color.Blue:
                        if (pieceNumber == 1) BluePiece1.SetPiecePosition(pieceBounds);
                        if (pieceNumber == 2) BluePiece2.SetPiecePosition(pieceBounds);
                        if (pieceNumber == 3) BluePiece3.SetPiecePosition(pieceBounds);
                        if (pieceNumber == 4) BluePiece4.SetPiecePosition(pieceBounds);
                        break;
                }
            }
        }

        /// <summary>
        /// Updates the piece text of all pieces on the board.
        /// </summary>
        private void UpdatePieceCount()
        {
            foreach (Piece piece in _engine.Board.Pieces)
            {
                int pieceCount = GetPieceCount(piece.Color, piece.Number);
                int pieceNumber = piece.Number + 1;

                switch(piece.Color)
                {
                    case Lokeiv.Games.Ludo.Model.Color.Red:
                        if (pieceNumber == 1) RedPiece1.PieceCount = pieceCount;
                        if (pieceNumber == 2) RedPiece2.PieceCount = pieceCount;
                        if (pieceNumber == 3) RedPiece3.PieceCount = pieceCount;
                        if (pieceNumber == 4) RedPiece4.PieceCount = pieceCount;
                        break;
                    case Lokeiv.Games.Ludo.Model.Color.Yellow:
                        if (pieceNumber == 1) YellowPiece1.PieceCount = pieceCount;
                        if (pieceNumber == 2) YellowPiece2.PieceCount = pieceCount;
                        if (pieceNumber == 3) YellowPiece3.PieceCount = pieceCount;
                        if (pieceNumber == 4) YellowPiece4.PieceCount = pieceCount;
                        break;
                    case Lokeiv.Games.Ludo.Model.Color.Green:
                        if (pieceNumber == 1) GreenPiece1.PieceCount = pieceCount;
                        if (pieceNumber == 2) GreenPiece2.PieceCount = pieceCount;
                        if (pieceNumber == 3) GreenPiece3.PieceCount = pieceCount;
                        if (pieceNumber == 4) GreenPiece4.PieceCount = pieceCount;
                        break;
                    case Lokeiv.Games.Ludo.Model.Color.Blue:
                        if (pieceNumber == 1) BluePiece1.PieceCount = pieceCount;
                        if (pieceNumber == 2) BluePiece2.PieceCount = pieceCount;
                        if (pieceNumber == 3) BluePiece3.PieceCount = pieceCount;
                        if (pieceNumber == 4) BluePiece4.PieceCount = pieceCount;
                        break;
                }
            }
        }

        private void SetNextAction(Lokeiv.Games.Ludo.Action nextAction, Player playersTurn)
        {
            string action = playersTurn.Name + " ";

            switch(nextAction)
            {
                case Lokeiv.Games.Ludo.Action.MovePiece:
                    action += "move piece";
                    break;
                case Lokeiv.Games.Ludo.Action.RollDie:
                    action += "roll die";
                    break;
            }

            ActionText = action;
        }

        /// <summary>
        /// Starts a new game.
        /// </summary>
        /// <param name="deserialize">Whether or not we are deserializing the ludo engine.</param>
        private void StartNewGameInternal(bool deserialize)
        {
            if (this._engine != null)
            {
                this._engine.DieRolled -= new DieRolledEventHandler(DieRolled);
                this._engine.PieceMoved -= new PieceMovedEventHandler(PieceMoved);
                this._engine.PlayerActionChanged -= new EventHandler<PlayerActionChangedEventArgs>(PlayerActionChanged);
            }

            // Initialize players from new game viewmodel
            InitializePlayers(deserialize);

            // Start a new game engine
            if (deserialize)
                this._engine = new Engine();
            else
            {
                this._engine = new Engine(GetPlayersFromPlayerViewModels(), this.NewGameViewModel.SettingsViewModel.GameOptions.Clone());
                this.SetFirstActionOfNewGame();
            }
            
            // Update piece labels after engine has been initialized
            this.UpdatePieceCount();

            this._engine.DieRolled += new DieRolledEventHandler(DieRolled);
            this._engine.PieceMoved += new PieceMovedEventHandler(PieceMoved);
            this._engine.PlayerActionChanged += new EventHandler<PlayerActionChangedEventArgs>(PlayerActionChanged);
            Die.LudoEngine = this._engine;
        }

        /// <summary>
        /// Updates player info controls to show the first action of a new game.
        /// </summary>
        private void SetFirstActionOfNewGame()
        {
            foreach (Player player in this._engine.Players)
                if (player.CurrentAction == Lokeiv.Games.Ludo.Action.RollDie)
                    this.Players[player.Color].PlayerAction = PlayerAction.RollDie;
        }

        #endregion  // Helper methods

        #region Event handlers
        
        /// <summary>
        /// Handles the <see cref="Engine.DieRolled"/> event to notify view about die value changes.        
        /// </summary>
        private void DieRolled(object sender, ActionEventArgs e)
        {            
            if (_dispatcher.CheckAccess())
            {                
                SetNextAction(e.Response.NextAction, e.Response.PlayersTurn);
                _validMoves = e.Response.MovablePieces;
            }
            else
            {
                _dispatcher.BeginInvoke(() =>
                {                    
                    SetNextAction(e.Response.NextAction, e.Response.PlayersTurn);
                    _validMoves = e.Response.MovablePieces;
                });
            }
        }

        /// <summary>
        /// Handles the <see cref="Engine.PieceMoved"/> event to notify view about position changes to piece.
        /// </summary>
        private void PieceMoved(object engine, ActionEventArgs e)
        {
            if (_dispatcher.CheckAccess())
            {
                this.UpdatePieceCount();
                this.UpdatePieceBounds(e.Response.MovablePieces);
                SetNextAction(e.Response.NextAction, e.Response.PlayersTurn);
            }
            else
            {
                _dispatcher.BeginInvoke(() =>
                    {
                        this.UpdatePieceCount();
                        this.UpdatePieceBounds(e.Response.MovablePieces);
                        SetNextAction(e.Response.NextAction, e.Response.PlayersTurn);
                    });
            }
        }

        /// <summary>
        /// Handles the <see cref="Engine.PlayerActionChanged"/> event to notify view about the current action of players.
        /// </summary>
        void PlayerActionChanged(object sender, PlayerActionChangedEventArgs e)
        {
            if (_dispatcher.CheckAccess())
            {
                this.PlayerActionChangedInternal(e);
            }
            else
            {
                _dispatcher.BeginInvoke(() =>
                    {
                        this.PlayerActionChangedInternal(e);
                    });
            }
        }

        private void PlayerActionChangedInternal(PlayerActionChangedEventArgs e)
        {
            PlayerAction playerAction = PlayerAction.Nothing;

            switch(e.Player.CurrentAction)
            {
                case Lokeiv.Games.Ludo.Action.RollDie:
                    playerAction = PlayerAction.RollDie;
                    break;
                case Lokeiv.Games.Ludo.Action.MovePiece:
                    playerAction = PlayerAction.MovePiece;
                    break;
            }

            if (e.Player.HasWon) playerAction = PlayerAction.Finished;

            switch (e.Player.Color)
            {
                case Lokeiv.Games.Ludo.Model.Color.Red:                    
                    this.RedPlayer.PlayerAction = playerAction;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.Yellow:
                    this.YellowPlayer.PlayerAction = playerAction;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.Green:
                    this.GreenPlayer.PlayerAction = playerAction;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.Blue:
                    this.BluePlayer.PlayerAction = playerAction;
                    break;
            }
        }

        #endregion  // Event handlers

        #region Commands

        /// <summary>
        /// Changes view to the new game view.
        /// </summary>
        private void OnNewGame()
        {
            this.NewGameViewModel.SettingsViewModel.GameOptions = this._engine.Options.Clone();

            if(NavigateToNewGame != null)                       
                NavigateToNewGame(this, EventArgs.Empty);                    
        }

        /// <summary>
        /// Exits the application.
        /// </summary>
        private void OnExit()
        {
            App.Quit();
        }

        /// <summary>
        /// Starts a new game when the user clicks the start button.
        /// </summary>
        /// <param name="deserialize">Whether or not we are deserializing the ludo engine.</param>
        private void OnStartNewGame()
        {
            this.StartNewGameInternal(false);
        }

        /// <summary>
        /// Handles a mouse click in the ludo board control. Checks if a <see cref="Piece"/> should be moved as a reaction to the mouse click.
        /// </summary>
        /// <param name="point"><see cref="Point"/> where the mouse was clicked.</param>
        public void HandleMouseClickInBoard(Point point)
        {
            List<Piece> piecesClicked = null;

            if (this.RedPlayer != null && this.RedPlayer.PlayerAction == PlayerAction.MovePiece)
            {
                piecesClicked = GetPiecesAtPosition(point, Lokeiv.Games.Ludo.Model.Color.Red);
            }
            else if (this.YellowPlayer != null && this.YellowPlayer.PlayerAction == PlayerAction.MovePiece)
            {
                piecesClicked = GetPiecesAtPosition(point, Lokeiv.Games.Ludo.Model.Color.Yellow);
            }
            else if (this.GreenPlayer != null && this.GreenPlayer.PlayerAction == PlayerAction.MovePiece)
            {
                piecesClicked = GetPiecesAtPosition(point, Lokeiv.Games.Ludo.Model.Color.Green);
            }
            else if (this.BluePlayer != null && this.BluePlayer.PlayerAction == PlayerAction.MovePiece)
            {
                piecesClicked = GetPiecesAtPosition(point, Lokeiv.Games.Ludo.Model.Color.Blue);
            }

            if (piecesClicked != null && piecesClicked.Count > 0 && _validMoves != null && _validMoves.Count > 0)
            {
                foreach (Piece pieceClicked in piecesClicked)
                {
                    foreach (MoveablePiece validMove in _validMoves)
                    {
                        if (validMove.Piece == pieceClicked)
                        {
                            this._engine.MovePiece(validMove);
                            return;
                        }
                    }
                }                
            }
        }

        #endregion

        #endregion  // Methods

        #region INotifyPropertyChanged
        /// <summary>
        /// Event to notify the view when a property has changed.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;        

        /// <summary>
        /// Fires the <see cref="PropertyChanged"/> event for the given <paramref name="propertyName"/>.
        /// </summary>
        /// <param name="propertyName">Name of property that has changed.</param>
        private void NotifyPropertyChanged(String propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion  // INotifyPropertyChanged

        #region ISerializable
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(System.Xml.XmlReader reader)
        {
            while (reader.Read())
            {
                if (reader.NodeType == System.Xml.XmlNodeType.Element)
                {
                    switch(reader.Name.ToLower())
                    {
                        case "engine":
                            // Read engine            
                            this.StartNewGameInternal(true);
                            this._engine.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            this.InitializePlayers(true);  // Loads player view models with the deserialized players
                            break;
                        case "view":
                            // Read view for the current piece coordinates                            
                            string view = reader.GetAttribute("Type");
                            if (!string.IsNullOrEmpty(view))
                            {
                                this.View = (Utils.BoardCoordinates.Views)Enum.Parse(typeof(Utils.BoardCoordinates.Views), view, true);
                            }
                            break;
                        case "newgameviewmodel":
                            this.NewGameViewModel.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "redpiece1":
                            this.RedPiece1.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "redpiece2":
                            this.RedPiece2.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "redpiece3":
                            this.RedPiece3.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "redpiece4":
                            this.RedPiece4.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "yellowpiece1":
                            this.YellowPiece1.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "yellowpiece2":
                            this.YellowPiece2.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "yellowpiece3":
                            this.YellowPiece3.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "yellowpiece4":
                            this.YellowPiece4.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "greenpiece1":
                            this.GreenPiece1.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "greenpiece2":
                            this.GreenPiece2.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "greenpiece3":
                            this.GreenPiece3.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "greenpiece4":
                            this.GreenPiece4.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "bluepiece1":
                            this.BluePiece1.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "bluepiece2":
                            this.BluePiece2.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "bluepiece3":
                            this.BluePiece3.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "bluepiece4":
                            this.BluePiece4.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                    }
                }
            }
        }

        public void WriteXml(System.Xml.XmlWriter writer)
        {
            // Engine should write itself
            writer.WriteStartElement("Engine");
            this._engine.WriteXml(writer);
            writer.WriteEndElement();

            // View for the current piece coordinates
            writer.WriteStartElement("View");
            writer.WriteAttributeString("Type", this.View.ToString());
            writer.WriteEndElement();

            // NewGameViewModel should write itself
            writer.WriteStartElement("NewGameViewModel");
            this.NewGameViewModel.WriteXml(writer);
            writer.WriteEndElement();

            // RedPiece1
            writer.WriteStartElement("RedPiece1");
            this.RedPiece1.WriteXml(writer);
            writer.WriteEndElement();

            // RedPiece2
            writer.WriteStartElement("RedPiece2");
            this.RedPiece2.WriteXml(writer);
            writer.WriteEndElement();

            // RedPiece3
            writer.WriteStartElement("RedPiece3");
            this.RedPiece3.WriteXml(writer);
            writer.WriteEndElement();

            // RedPiece4
            writer.WriteStartElement("RedPiece4");
            this.RedPiece4.WriteXml(writer);
            writer.WriteEndElement();

            // YellowPiece1
            writer.WriteStartElement("YellowPiece1");
            this.YellowPiece1.WriteXml(writer);
            writer.WriteEndElement();

            // YellowPiece2
            writer.WriteStartElement("YellowPiece2");
            this.YellowPiece2.WriteXml(writer);
            writer.WriteEndElement();

            // YellowPiece3
            writer.WriteStartElement("YellowPiece3");
            this.YellowPiece3.WriteXml(writer);
            writer.WriteEndElement();

            // YellowPiece4
            writer.WriteStartElement("YellowPiece4");
            this.YellowPiece4.WriteXml(writer);
            writer.WriteEndElement();

            // GreenPiece1
            writer.WriteStartElement("GreenPiece1");
            this.GreenPiece1.WriteXml(writer);
            writer.WriteEndElement();

            // GreenPiece2
            writer.WriteStartElement("GreenPiece2");
            this.GreenPiece2.WriteXml(writer);
            writer.WriteEndElement();

            // GreenPiece3
            writer.WriteStartElement("GreenPiece3");
            this.GreenPiece3.WriteXml(writer);
            writer.WriteEndElement();

            // GreenPiece4
            writer.WriteStartElement("GreenPiece4");
            this.GreenPiece4.WriteXml(writer);
            writer.WriteEndElement();

            // BluePiece1
            writer.WriteStartElement("BluePiece1");
            this.BluePiece1.WriteXml(writer);
            writer.WriteEndElement();

            // BluePiece2
            writer.WriteStartElement("BluePiece2");
            this.BluePiece2.WriteXml(writer);
            writer.WriteEndElement();

            // BluePiece3
            writer.WriteStartElement("BluePiece3");
            this.BluePiece3.WriteXml(writer);
            writer.WriteEndElement();

            // BluePiece4
            writer.WriteStartElement("BluePiece4");
            this.BluePiece4.WriteXml(writer);
            writer.WriteEndElement();

            // RedPlayer
            writer.WriteStartElement("RedPlayer");
            this.RedPlayer.WriteXml(writer);
            writer.WriteEndElement();

            // YellowPlayer
            writer.WriteStartElement("YellowPlayer");
            this.YellowPlayer.WriteXml(writer);
            writer.WriteEndElement();

            // GreenPlayer
            writer.WriteStartElement("GreenPlayer");
            this.GreenPlayer.WriteXml(writer);
            writer.WriteEndElement();

            // BluePlayer
            writer.WriteStartElement("BluePlayer");
            this.BluePlayer.WriteXml(writer);
            writer.WriteEndElement();
        }
        #endregion
    }
}