﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Lokeiv.Games.Ludo.Model;
using System.ComponentModel;

namespace Ludo.WP7.Client.ViewModels
{
    public class SettingsViewModel : INotifyPropertyChanged
    {

        #region Properties

        private GameOptions _gameOptions;
        public GameOptions GameOptions
        {
            get { return this._gameOptions; }
            set
            {
                if (this._gameOptions != value)
                {
                    this._gameOptions = value;
                    NotifyPropertyChanged("GameOptions");
                }
            }
        }

        public GameSpeed GameSpeed
        {
            get { return this.GameOptions.GameSpeed; }
            set
            {
                if (this.GameOptions.GameSpeed != value)
                {
                    this.GameOptions.GameSpeed = value;
                    NotifyPropertyChanged("GameSpeed");
                    NotifyPropertyChanged("IsGameSpeedSlowChecked");
                    NotifyPropertyChanged("IsGameSpeedMediumChecked");
                    NotifyPropertyChanged("IsGameSpeedFastChecked");
                }
            }
        }

        public bool IsGameSpeedSlowChecked
        {
            get { return this.GameSpeed == Lokeiv.Games.Ludo.Model.GameSpeed.Slow; }
            set
            {
                if (value == true && this.GameSpeed != Lokeiv.Games.Ludo.Model.GameSpeed.Slow)
                    this.GameSpeed = Lokeiv.Games.Ludo.Model.GameSpeed.Slow;                
            }
        }

        public bool IsGameSpeedMediumChecked
        {
            get { return this.GameSpeed == Lokeiv.Games.Ludo.Model.GameSpeed.Medium; }
            set
            {
                if (value == true && this.GameSpeed != Lokeiv.Games.Ludo.Model.GameSpeed.Medium)
                    this.GameSpeed = Lokeiv.Games.Ludo.Model.GameSpeed.Medium;
            }
        }

        public bool IsGameSpeedFastChecked
        {
            get { return this.GameSpeed == Lokeiv.Games.Ludo.Model.GameSpeed.Fast; }
            set
            {
                if (value == true && this.GameSpeed != Lokeiv.Games.Ludo.Model.GameSpeed.Fast)
                    this.GameSpeed = Lokeiv.Games.Ludo.Model.GameSpeed.Fast;
            }
        }
        #endregion  // Properties

        #region Constructors

        public SettingsViewModel()
        {
            this.GameOptions = new GameOptions();
        }

        public SettingsViewModel(GameOptions gameOptions)
        {
            this.GameOptions = gameOptions;
        }

        #endregion  // Constructors

        #region Methods

        #endregion  // Methods
        
        #region INotifyPropertyChanged
        /// <summary>
        /// Event to notify the view when a property has changed.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Fires the <see cref="PropertyChanged"/> event for the given <paramref name="propertyName"/>.
        /// </summary>
        /// <param name="propertyName">Name of property that has changed.</param>
        private void NotifyPropertyChanged(String propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion  // INotifyPropertyChanged

    }
}
