﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Lokeiv.Games.Ludo.Service
{
    class InMemoryStateProvider : IStateProvider
    {
        private GameRoom _gameRoom;

        #region Singleton Pattern
        private static InMemoryStateProvider _instance;
        public static InMemoryStateProvider Instance
        {
            get
            {
                if (_instance == null)
                    _instance = new InMemoryStateProvider();

                return _instance;
            }
        }

        private InMemoryStateProvider()
        {

        }
        #endregion // Singleton Pattern

        #region IStateProvider
        public GameRoom GetGameRoom()
        {
            if (this._gameRoom == null)
                this._gameRoom = new GameRoom();

            return this._gameRoom;
        }

        public Table GetTable(string name)
        {
            GameRoom room = this.GetGameRoom();

            foreach (Table table in room.Tables)
                if (string.Equals(table.Name, name))
                    return table;

            return null;
        }


        public Client GetClient(string nickName)
        {
            GameRoom room = this.GetGameRoom();

            foreach (Client client in room.Clients)
                if (string.Equals(client.NickName, nickName))
                    return client;

            return null;
        }
        #endregion // IStateProvider
    }
}
