using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;

namespace Lokeiv.Games.Dice
{
    public abstract class Die : IXmlSerializable
    {
        protected DieValue _value = null;
        protected Random _randomizer = null;

        # region Public properties

        public DieValue Value
        {
            get { return this._value; }
            set 
            {
                if (this._value != value)
                {
                    this._value = value;
                    this.OnDieValueChanged(EventArgs.Empty);
                }                
            }
        }

        #endregion

        #region Public events

        public delegate void DieRolledEventHandler(object die, DieRolledEventArgs e);

        public event DieRolledEventHandler DieRolled;

        protected void OnDieRolled(object die, DieRolledEventArgs e)
        {
            if (this.DieRolled != null)
            {
                this.DieRolled(this, e);
            }
        }

        public event EventHandler DieValueChanged;

        protected void OnDieValueChanged(EventArgs e)
        {
            if (this.DieValueChanged != null)
                this.DieValueChanged(this, e);
        }

        #endregion

        #region Constructors

        public Die()
        {            
            this._randomizer = new Random();
        }

        #endregion

        protected abstract DieValue GetNewRandomValue();

        public DieValue Roll()
        {
            this._value = this.GetNewRandomValue();
            this.OnDieRolled(this, new DieRolledEventArgs(this._value));
            return this._value;
        }

        #region IXmlSerializable        
        public virtual System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public virtual void ReadXml(System.Xml.XmlReader reader)
        {
            throw new NotImplementedException();
        }

        public virtual void WriteXml(System.Xml.XmlWriter writer)
        {
            if(this.Value != null) this.Value.WriteXml(writer);
        }
        #endregion
    }
}
