using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;
using Ascend;
using Ascend.Windows.Forms;

namespace Lokeiv.Games.Ludo.Forms.Client
{
    class BoardRender : GradientBackgroundRender
    {
        private float _dotSize = 4;
        private int _posOffset = 2;
        private SolidBrush _foreBrush;
        private Color _foreColor = Color.Yellow;

        /// <summary>
        /// Occurs when the ForeColor property changes.
        /// </summary>
        public event EventHandler ForeColorChanged;

        #region Public properties
        /// <summary>
        /// The size of the dots on the die.
        /// </summary>
        public float DotSize
        {
            get { return _dotSize; }
            set { _dotSize = value; }
        }

        /// <summary>
        /// The color assoicated with the text.
        /// </summary>
        public Color ForeColor
        {
            get
            {
                return this._foreColor;
            }

            set
            {
                if (value == this._foreColor)
                {
                    return;
                }

                this._foreColor = value;
                this.CreateForeBrush();
                this.OnForeColorChanged(new EventArgs());
            }
        }

        #endregion

        #region Contructors
        /// <summary>
        /// Initializes a new instance of the <see cref="BoardRender"/> class.
        /// </summary>
        public BoardRender()
            : base()
        {
            base.Border = new Border(1);
            base.BorderColor = new BorderColor(Color.Black);
        }

        #endregion

        /// <summary>
        /// Raises the ForeColorChanged event. 
        /// </summary>
        /// <param name="e">An System.EventArgs that contains the event data.</param>
        protected virtual void OnForeColorChanged(System.EventArgs e)
        {
            if (this.ForeColorChanged != null)
            {
                this.ForeColorChanged(this, e);
            }
        }

        /// <summary>
        /// Draws the board control.
        /// </summary>
        /// <param name="e">A System.Windows.Forms.PaintEventArgs that contains the event data.</param>
        public override void Render(System.Windows.Forms.PaintEventArgs e)
        {
            if (e == null)
            {
                throw new ArgumentNullException("e");
            }

            RectangleF bounds;
            float topOffset = (this._posOffset + base.Padding.Top + this.Border.Top);
            float leftPosition = (this.DisplayRectangle.Left + base.Padding.Left + this._posOffset);
            float topPosition = (this.DisplayRectangle.Top + this._posOffset + base.Padding.Top + this.Border.Top);
            float rightOffset = (this._posOffset + base.Padding.Right + this.Border.Right);
            float bottomOffset = (this._posOffset + base.Padding.Bottom + this.Border.Bottom);
            bounds = new RectangleF(leftPosition, topPosition, (this.DisplayRectangle.Width - (leftPosition + rightOffset)), (this.DisplayRectangle.Height - (topOffset + bottomOffset)));

            if (this.DisplayRectangle.Height > 0 && this.DisplayRectangle.Width > 0)
            {
                base.Render(e);
                this.CreateForeBrush();

                    if (!bounds.IsEmpty)
                    {
                    }
            }
        }

        /// <summary>
        /// Creates the solid brush needed to draw the foreground.
        /// </summary>
        protected virtual void CreateForeBrush()
        {
            if (base.Enabled)
            {
                this._foreBrush = new SolidBrush(this._foreColor);
            }
            else
            {
                this._foreBrush = new SolidBrush(SystemColors.GrayText);
            }
        }
    }
}
