using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Lokeiv.Games.Ludo.Model;

namespace Lokeiv.Games.Ludo.Forms.Client
{
    public partial class Game : Form, IGame
    {
        private NewGameForm _newGameForm = null;
        private Ludo _ludo;

        /// <summary>
        /// Property to manipulate the ludo game used in this form.
        /// Used by test cases.
        /// </summary>
        public Ludo Ludo
        {
            get { return this._ludo; }
            set { this._ludo = value; }
        }

        public Game()
        {
            InitializeComponent();
            this._newGameForm = new NewGameForm();
        }

        private void StartNewGame()
        {            
            switch(this._newGameForm.ShowDialog(this))
            {
                case DialogResult.OK:
                    List<Player> players = this._newGameForm.Players;
                    this.PrepareGame(players);

                    //this.gameEngine.RunWorkerAsync(players);
                    //this.board1.StartNewGame(numberOfPlayers);
                    break;
                default:
                    break;
            }
        }

        public void PrepareGame(List<Player> players)
        {
            // Order list of players by color
            Player.SortPlayersList(players);
            this._ludo = new Ludo(players, this.die1.DieModel);
            this.board1.SetGameModel(this._ludo.Board, this);
            this.InitializePlayers();
            this._ludo.StartGame();
            this.gameInstructionsLabel.Text = String.Format(Properties.Resources.InstructionRollDie, this._ludo.LastResponse.PlayersTurn.Name);
        }
        

        /// <summary>
        /// Updates players in GUI.
        /// </summary>
        private void InitializePlayers()
        {            
            int defaultImageIndex = 9;
            this.redPlayerCaption.ImageIndex = defaultImageIndex;
            this.redPlayerCaption.Text = "Red player";
            this.yellowPlayerCaption.ImageIndex = defaultImageIndex;
            this.yellowPlayerCaption.Text = "Yellow player";
            this.greenPlayerCaption.ImageIndex = defaultImageIndex;
            this.greenPlayerCaption.Text = "Green player";
            this.bluePlayerCaption.ImageIndex = defaultImageIndex;
            this.bluePlayerCaption.Text = "Blue player";

            // Set the right image and caption
            foreach (Player player in this._ludo.Players)
            {
                switch (player.Color)
                {
                    case Color.Red:
                        this.redPlayerCaption.Text = player.Name;

                        if (player.Type == PlayerType.Computer)
                            this.redPlayerCaption.ImageIndex = 5;
                        else
                            this.redPlayerCaption.ImageIndex = 0;
                        break;
                    case Color.Yellow:
                        this.yellowPlayerCaption.Text = player.Name;

                        if (player.Type == PlayerType.Computer)
                            this.yellowPlayerCaption.ImageIndex = 6;
                        else
                            this.yellowPlayerCaption.ImageIndex = 1;
                        break;
                    case Color.Green:
                        this.greenPlayerCaption.Text = player.Name;

                        if (player.Type == PlayerType.Computer)
                            this.greenPlayerCaption.ImageIndex = 7;
                        else
                            this.greenPlayerCaption.ImageIndex = 2;
                        break;
                    case Color.Blue:
                        this.bluePlayerCaption.Text = player.Name;

                        if (player.Type == PlayerType.Computer)
                            this.bluePlayerCaption.ImageIndex = 8;
                        else
                            this.bluePlayerCaption.ImageIndex = 3;
                        break;
                }
            }            
        }

        private void UpdatePlayersTurn(Player player)
        {
            this.redPlayerCaption.Enabled = false;
            this.yellowPlayerCaption.Enabled = false;
            this.greenPlayerCaption.Enabled = false;
            this.bluePlayerCaption.Enabled = false;
            
            if (player.Name == this.redPlayerCaption.Text)
            {
                this.redPlayerCaption.Enabled = true;
            }
            else if (player.Name == this.yellowPlayerCaption.Text)
            {
                this.yellowPlayerCaption.Enabled = true;
            }
            else if (player.Name == this.greenPlayerCaption.Text)
            {
                this.greenPlayerCaption.Enabled = true;
            }
            else if (player.Name == this.bluePlayerCaption.Text)
            {
                this.bluePlayerCaption.Enabled = true;
            }
        }

        public void MovePiece(MoveablePiece moveablePiece)
        {
            Response response = this._ludo.MovePiece(moveablePiece);

            if (response != null)
            {
                this.UpdatePlayersTurn(response.PlayersTurn);
                string text = "";

                if (response.PlayerWon != null)
                {
                    text += String.Format(Properties.Resources.InstructionHasFinished + (char)13, response.PlayerWon.Name);
                }

                if (response.NextAction == Action.RollDie)
                {
                    text += String.Format(Properties.Resources.InstructionRollDie, response.PlayersTurn.Name);
                }

                this.gameInstructionsLabel.Text = text;                
            }

            this.board1.Invalidate();
        }

        #region Event handlers
        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.StartNewGame();
        }

        private void Game_Shown(object sender, EventArgs e)
        {
            this.StartNewGame();
        }

        private void Game_FormClosed(object sender, FormClosedEventArgs e)
        {
            Application.Exit();
        }

        #endregion

        private void gameEngine_DoWork(object sender, DoWorkEventArgs e)
        {
            List<Player> players = (List<Player>)e.Argument;
            Ludo ludo = new Ludo(players, this.die1.DieModel);
            this.board1.SetGameModel(ludo.Board, this);
            ludo.StartGame();

            while(!ludo.areAllFinished())
            {
                //gameEngine.ReportProgress(0, ludo.MovePiece(new MoveablePiece()));
            }
        }

        private void gameEngine_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            Response response = e.UserState as Response;
            this.die1.DieValue = response.DieValue;
            this.board1.Invalidate();
        }

        private void die1_Click(object sender, EventArgs e)
        {
            Response response = this._ludo.RollDie();

            if (response != null)
            {
                this.UpdatePlayersTurn(response.PlayersTurn);
                string text = "";

                if (response.NextAction == Action.MovePiece)
                {
                    text += String.Format(Properties.Resources.InstructionMovePiece, response.PlayersTurn.Name);
                }
                else if (response.NextAction == Action.RollDie)
                {
                    text += String.Format(Properties.Resources.InstructionRollDie, response.PlayersTurn.Name);
                }

                this.gameInstructionsLabel.Text = text;
            }
        }

    }
}