using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;
using Ascend;
using Ascend.Windows.Forms;
using Lokeiv.Games.Ludo.Model;
using Lokeiv.Rendering;

namespace Lokeiv.Games.Ludo.Forms.Client
{
    public class PieceRender : GradientBackgroundRender
    {
        private List<Piece> _pieces = new List<Piece>();
        private int _posOffset = 0;
        private RectangleF _pieceBounds;
        private Border _border;
        private System.Drawing.Color _adjustedBorderColorTop;
        private IBoard _board;

        public Piece Piece
        {
            get 
            {
                if(_pieces.Count > 0)
                    return _pieces[0];

                return null;
            }
        }

        public List<Piece> Pieces
        {
            get { return _pieces; }
        }

        public new Border Border
        {
            get { return _border; }
            set { _border = value; }
        }

        public RectangleF PieceBounds
        {
            get { return _pieceBounds; }
            set { _pieceBounds = value; this.DisplayRectangle = new Rectangle((int)value.X, (int)value.Y, (int)value.Width, (int)value.Height); }
        }

        public PieceRender(Lokeiv.Games.Ludo.Model.Piece piece, IBoard board)
        {
            this.Padding = new System.Windows.Forms.Padding(0);
            this.Border = new Border(2);
            this.BorderColor = new BorderColor(System.Drawing.Color.Black);
            this._pieces.Add(piece);
            piece.SquareChanged += this.PieceModel_SquareChanged;
            this._board = board;


            // Select the color to draw this house with
            switch (piece.Color)
            {
                case Lokeiv.Games.Ludo.Model.Color.Red:
                    this.GradientHighColor = System.Drawing.Color.LightCoral;
                    this.GradientLowColor = System.Drawing.Color.DarkRed;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.Yellow:
                    this.GradientHighColor = System.Drawing.Color.LightYellow;
                    this.GradientLowColor = System.Drawing.Color.Yellow;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.Green:
                    this.GradientHighColor = System.Drawing.Color.LightGreen;
                    this.GradientLowColor = System.Drawing.Color.DarkGreen;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.Blue:
                    this.GradientHighColor = System.Drawing.Color.LightBlue;
                    this.GradientLowColor = System.Drawing.Color.DarkBlue;
                    break;
            }  
        }

        public override void Render(System.Windows.Forms.PaintEventArgs e)
        {
            if (e == null)
            {
                throw new ArgumentNullException("e");
            }

            RectangleF bounds;
            //float topOffset = (this._posOffset + base.Padding.Top);
            //float leftPosition = (this.DisplayRectangle.Left + base.Padding.Left + this._posOffset);
            //float topPosition = (this.DisplayRectangle.Top + this._posOffset + base.Padding.Top);
            //float rightOffset = (this._posOffset + base.Padding.Right);
            //float bottomOffset = (this._posOffset + base.Padding.Bottom);
            //bounds = new RectangleF(leftPosition, topPosition, (this.DisplayRectangle.Width - (rightOffset)), (this.DisplayRectangle.Height - (topOffset + bottomOffset)));
            bounds = new RectangleF(this.DisplayRectangle.Left, this.DisplayRectangle.Top, this.DisplayRectangle.Width, this.DisplayRectangle.Height);
            PointF triangleTopPoint = new PointF(0, 0);
                        
            if (this.DisplayRectangle.Height > 0 && this.DisplayRectangle.Width > 0)
            {
                this.CreateBackBrush();

                // Draw background color
                e.Graphics.FillEllipse(this.BackBrush, this.PieceBounds);

                // Draw number of pieces if more than one
                if (this.Pieces.Count > 1)
                {
                    StringRendering.DrawTextCentered(e.Graphics, Brushes.Black, new Font("Sans Serif", 10), this.Pieces.Count.ToString(), this.PieceBounds);
                }
            }

            // Draw border
            if (this._border.Top > 0)
            {
                this._adjustedBorderColorTop = this.BorderColor.Top;
                Pen borderPen = new Pen(this._adjustedBorderColorTop, this._border.Top);
                try
                {
                    //float offset = 0; // (float)(this._border.Top / 2.0);
                    e.Graphics.DrawEllipse(borderPen, this.PieceBounds);
                }
                catch
                {
                    throw;
                }
                finally
                {
                    borderPen.Dispose();
                }
            }
        }

        private void PieceModel_SquareChanged(object sender, EventArgs e)
        {
            Piece piece = sender as Piece;
            SquareRender square = this._board.Squares[piece.Square.Number];
            RectangleF pieceBounds = new RectangleF(square.DisplayRectangle.X + square.Padding.Left, square.DisplayRectangle.Y + square.Padding.Top, this.PieceBounds.Width, this.PieceBounds.Height);
            this.PieceBounds = pieceBounds;
        }
        
    }
}
