using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using Ascend;
using Ascend.Windows.Forms;
using Lokeiv.Games.Ludo.Model;

namespace Lokeiv.Games.Ludo.Forms.Client
{
    public class SquareRender : GradientBackgroundRender
    {
        private int _posOffset = 2;
        private Square _square;
        private RectangleF _bounds;

        public RectangleF Bounds
        {
            get { return _bounds; }
            set { _bounds = value; this.DisplayRectangle = new Rectangle((int)value.X, (int)value.Y, (int)value.Width, (int)value.Height); }
        }
	
        public Square Square
        {
            get { return _square; }
            set { _square = value; }
        }

        public SquareRender(Square square)
        {
            this._square = square;
            this.Padding = new Padding(4);
            this.Border = new Border(1);
            this.BorderColor = new BorderColor(System.Drawing.Color.Black);

            // Select the color to draw this square with
            switch (this._square.Color)
            {
                case Lokeiv.Games.Ludo.Model.Color.Red:
                    this.GradientHighColor = System.Drawing.Color.LightCoral;
                    this.GradientLowColor = System.Drawing.Color.DarkRed;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.Yellow:
                    this.GradientHighColor = System.Drawing.Color.LightYellow;
                    this.GradientLowColor = System.Drawing.Color.Yellow;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.Green:
                    this.GradientHighColor = System.Drawing.Color.LightGreen;
                    this.GradientLowColor = System.Drawing.Color.DarkGreen;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.Blue:
                    this.GradientHighColor = System.Drawing.Color.LightBlue;
                    this.GradientLowColor = System.Drawing.Color.DarkBlue;
                    break;
                case Lokeiv.Games.Ludo.Model.Color.White:
                    this.GradientHighColor = System.Drawing.Color.White;
                    this.GradientLowColor = System.Drawing.Color.WhiteSmoke;
                    break;
            }            

            // The squares in the goals should not have a border
            //switch (this._square.Number)
            //{
            //    case 57:
            //    case 63:
            //    case 69:
            //    case 75:
            //        this.Border = new Border(0);
            //        break;
            //}
        }

        public override void Render(System.Windows.Forms.PaintEventArgs e)
        {
            if (e == null)
            {
                throw new ArgumentNullException("e");
            }

            RectangleF bounds;
            float topOffset = (this._posOffset + base.Padding.Top + this.Border.Top);
            float leftPosition = (this.DisplayRectangle.Left + base.Padding.Left + this._posOffset);
            float topPosition = (this.DisplayRectangle.Top + this._posOffset + base.Padding.Top + this.Border.Top);
            float rightOffset = (this._posOffset + base.Padding.Right + this.Border.Right);
            float bottomOffset = (this._posOffset + base.Padding.Bottom + this.Border.Bottom);
            bounds = new RectangleF(leftPosition, topPosition, (this.DisplayRectangle.Width - (leftPosition + rightOffset)), (this.DisplayRectangle.Height - (topOffset + bottomOffset)));

            if (this.DisplayRectangle.Height > 0 && this.DisplayRectangle.Width > 0)
            {
                base.Render(e);                
                //e.Graphics.DrawString(this._square.Number.ToString(), new Font("Courier", 10), Brushes.Black, bounds.Left, this.Bounds.Top);
            }
        }
    }
}
