using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Lokeiv.Games.Ludo.Forms.InternetGame;
using Lokeiv.Rendering;

namespace Lokeiv.Games.Ludo.Forms.Client
{
    public partial class StartupForm : Form
    {
        public StartupForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Updates the help text that describes the type of game chosen.
        /// </summary>
        private void GameTypeChanged()
        {
            string text = "";

            if (internetRadioButton.Checked)
            {
                text = Properties.Resources.DesciptionInternetGame;
            }
            else
            {
                text = Properties.Resources.DescriptionLocalGame;
            }

            this.helpCaption.Text = text;
        }

        /// <summary>
        /// Continue to game room or start a single client game.
        /// </summary>
        private void Start()
        {
            if (localRadioButton.Checked)
            {
                if (this.glassCheckBox.Checked)
                {
                    new GameGlass().Show();
                }
                else
                {
                    new Game().Show();
                }

                this.Close();
            }
            else if (internetRadioButton.Checked)
            {
                new LudoRoom().Show();
                this.Close();
            }
        }

        /// <summary>
        /// Checks whether Aero is available, i.e. Windows version is 6 or greater
        /// and current theme is not Aero Basic.
        /// </summary>
        /// <returns>True if Aero is available</returns>
        private bool AeroAvailable()
        {
            OperatingSystem os = Environment.OSVersion;

            if (os.Version.Major < 6)
            {
                return false;
            }            

            if (!DesktopWindowManagerApi.DwmIsCompositionEnabled())
            {
                return false;
            }

            return true;
        }

        private void internetRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            this.GameTypeChanged();
        }

        private void startButton_Click(object sender, EventArgs e)
        {
            this.Start();
        }

        private void StartupForm_Load(object sender, EventArgs e)
        {
            this.glassCheckBox.Enabled = this.AeroAvailable();
            this.glassCheckBox.Checked = false;
        }
    }
}