using System;
using System.Collections.Generic;
using System.Text;
using Lokeiv.Games.Ludo.Model;

namespace Lokeiv.Games.Ludo.Model.ComputerPlayer
{
    class CompositeBestForPlayer : IMoveStrategy 
    {
        private List<IMoveStrategy> _moves = new List<IMoveStrategy>();

        #region Constructors
        public CompositeBestForPlayer()
        {

        }
        #endregion

        #region Private Methods
        
        /// <summary>
        /// Creates the right type of strategy and adds it to the list of moves.
        /// </summary>
        /// <param name="moveable"><see cref="MoveablePiece"/> representing the move</param>
        /// <param name="moveType">Type of move</param>
        private void CreateStrategy(MoveablePiece moveable, Moves moveType)
        {
            switch (moveType)
            {
                case Moves.CreateBlock:
                    this._moves.Add(new CreateBlockStrategy(moveable));
                    break;
                case Moves.Generic:
                    this._moves.Add(new GenericMoveStrategy(moveable));
                    break;
                case Moves.OutOfHouse:
                    this._moves.Add(new OutOfHouseStrategy(moveable));
                    break;
                case Moves.RemoveBlock:
                    this._moves.Add(new RemoveBlockStrategy(moveable));
                    break;
                case Moves.ToGoal:
                    this._moves.Add(new ToGoalStrategy(moveable));
                    break;
                case Moves.ToSecureSquare:
                    this._moves.Add(new ToSecureSquareStrategy(moveable));
                    break;
                case Moves.TurnIn:
                    this._moves.Add(new TurnInStrategy(moveable));
                    break;
            }
        }
        
        #endregion

        #region Public Methods

        public void AddCorrectMoveStrategy(MoveablePiece movable)
        {
            // Create all possible strategies for the move
            if (movable.Piece.Square.IsHouse && movable.Square.IsCrossing)
            {
                // Move out of hose
                this.CreateStrategy(movable, Moves.OutOfHouse);
            }
            
            if(movable.Square.Pieces.Count > 0)
            {
                // Other pieces on the destination square
                bool onlySameColor = true;
                bool onlyOpponentColor = true;
                bool isExistingOwnBlock = false;
                bool isExitingOpponentBlock = false;
                int numberOfOwnPieces = 0;
                int numberOfOpponentPieces = 0;

                foreach (Piece piece in movable.Square.Pieces)
                {
                    
                }
            }

        //            TurnIn = 0, 
        //OutOfHouse = 1, 
        //CreateBlock = 2, 
        //ToSecureSquare = 3, 
        //ToGoal = 4, 
        //Generic = 5, 
        //RemoveBlock = 6
        }

        public MoveablePiece GetBestMove()
        {
            IMoveStrategy bestMove = null;

            if (this._moves == null)
                return null;

            // For each possible move, choose the best one
            foreach (IMoveStrategy move in this._moves)
            {
                if (move.GetType().Equals(typeof(TurnInStrategy)))  // The best move
                {
                    bestMove = move;
                }
                else if (move.GetType().Equals(typeof(OutOfHouseStrategy)))  // The second best move, and so on...
                {
                    if (!bestMove.GetType().Equals(typeof(TurnInStrategy)))
                        bestMove = move;
                }
                else if (move.GetType().Equals(typeof(CreateBlockStrategy)))
                {
                    if (!(bestMove.GetType().Equals(typeof(TurnInStrategy)) || bestMove.GetType().Equals(typeof(OutOfHouseStrategy))))
                        bestMove = move;
                }
                else if (move.GetType().Equals(typeof(ToSecureSquareStrategy)))
                {
                    if (!(bestMove.GetType().Equals(typeof(TurnInStrategy)) || bestMove.GetType().Equals(typeof(OutOfHouseStrategy))
                        || bestMove.GetType().Equals(typeof(CreateBlockStrategy))))
                        bestMove = move;
                }
                else if (move.GetType().Equals(typeof(ToGoalStrategy)))
                {
                    if (!(bestMove.GetType().Equals(typeof(TurnInStrategy)) || bestMove.GetType().Equals(typeof(OutOfHouseStrategy))
                        || bestMove.GetType().Equals(typeof(CreateBlockStrategy)) || bestMove.GetType().Equals(typeof(ToSecureSquareStrategy))))
                        bestMove = move;
                }
                else if (move.GetType().Equals(typeof(GenericMoveStrategy)))
                {
                    if (!(bestMove.GetType().Equals(typeof(TurnInStrategy)) || bestMove.GetType().Equals(typeof(OutOfHouseStrategy))
                        || bestMove.GetType().Equals(typeof(CreateBlockStrategy)) || bestMove.GetType().Equals(typeof(ToSecureSquareStrategy))
                        || bestMove.GetType().Equals(typeof(ToGoalStrategy))))
                        bestMove = move;
                }
                else if (move.GetType().Equals(typeof(RemoveBlockStrategy)))  // The worst move
                {
                    if (!(bestMove.GetType().Equals(typeof(TurnInStrategy)) || bestMove.GetType().Equals(typeof(OutOfHouseStrategy))
                        || bestMove.GetType().Equals(typeof(CreateBlockStrategy)) || bestMove.GetType().Equals(typeof(ToSecureSquareStrategy))
                        || bestMove.GetType().Equals(typeof(ToGoalStrategy)) || bestMove.GetType().Equals(typeof(RemoveBlockStrategy))))
                        bestMove = move;
                }
            }  // foreach
            
            return bestMove.MovablePiece;
        }

        #endregion

        #region IMoveStrategy Members

        public MoveablePiece MovablePiece
        {
            get 
            {
                return this.GetBestMove();
            }
        }

        #endregion
    }
}
