using System;
using System.Collections.Generic;
using System.Text;
using System.Transactions;

namespace Lokeiv.Games.Ludo.Model
{
    public class Piece
    {
        private Color _color = Color.White;
        private Boolean _readyForFinish = false;
        private Square _square = null;

        public EventHandler SquareChanged;

        #region Properties
        public Color Color
        {
            get { return this._color; }
        }

        public Boolean IsReadyForFinish
        {
            get { return this._readyForFinish; }
        }

        public Square Square
        {
            get { return this._square; }
        }
        #endregion

        #region Constructors
        internal Piece(Color color)
        {
            this._color = color;            
        }
        #endregion

        protected void OnSquareChanged(EventArgs e)
        {
            if (this.SquareChanged != null)
            {
                this.SquareChanged(this, e);
            }
        }

        #region Internal Methods
        internal void SetReadyForFinish(Boolean isReadyForFinish)
        {
            this._readyForFinish = isReadyForFinish;
        }

        /// <summary>
        /// Sets the <see cref="Square"/> that this <see cref="Piece"/> is at, 
        /// and updates the <see cref="Square"/> about the <see cref="Piece"/> that is on it.
        /// </summary>
        /// <param name="square">The <see cref="Square"/> that this <see cref="Piece"/> is at</param>
        internal void SetSquare(Square square)
        {
            // Todo: Run this code within a transaction
            if (this._square != null)
                this._square.Pieces.Remove(this);

            this._square = square;
            this._square.Pieces.Add(this);
            this.OnSquareChanged(EventArgs.Empty);
        }
        #endregion

    }
}
