using System;
using System.Collections.Generic;
using System.Text;

namespace Lokeiv.Games.Ludo.Model
{
    public class Player
    {
        private PlayerType _type = PlayerType.Human;
        private String _name = String.Empty;
        private Color _color = Color.White;
        private Boolean _hasWon = false;
        private int _numberOfMovesLeft = 0;

        #region Properties
        public PlayerType Type
        {
            get { return this._type; }
            set { this._type = value; }
        }

        public String Name
        {
            get { return this._name; }
            set { this._name = value; }
        }

        public Color Color
        {
            get { return this._color; }
        }

        public Boolean HasWon
        {
            get { return this._hasWon; }
        }

        public int NumberOfMovesLeft
        {
            get { return this._numberOfMovesLeft; }
        }
        #endregion

        #region Contructors
        public Player(String name, PlayerType type, Color color)
        {
            this._name = name;
            this._type = type;
            this._color = color;
            this._hasWon = false;
            this._numberOfMovesLeft = 59;
        }
        #endregion

        #region Internal Methods
        internal void SetColor(Color color)
        {
            this._color = color;
        }

        internal void SetNumberOfMovesLeft(int numberOfMovesLeft)
        {
            this._numberOfMovesLeft = numberOfMovesLeft;
        }
        #endregion

        /// <summary>
        /// Sets if the player has finished.
        /// </summary>
        /// <param name="hasWon"><see cref="Boolean"/></param>
        public void SetWon(Boolean hasWon)
        {
            this._hasWon = hasWon;
        }

        /// <summary>
        /// Sorts a list of players using the <see cref="Comparison">CompareByColor</see>.
        /// </summary>
        /// <param name="players">The list of players to sort</param>
        public static void SortPlayersList(List<Player> players)
        {
            Comparison<Player> compareByColor = new Comparison<Player>(Player.CompareByColor);
            players.Sort(compareByColor);
        }

        /// <summary>
        /// Used to create a new <see cref="Comparison"/> of <see cref="Player"/>, for
        /// sorting a list of players by color.
        /// </summary>
        /// <param name="player1"><see cref="Player"/></param>
        /// <param name="player2"><see cref="Player"/></param>
        /// <returns><see cref="int"/></returns>
        public static int CompareByColor(Player player1, Player player2)
        {
            if (player1.Color == player2.Color)
                return 0;  // Equal

            if (player1.Color < player2.Color)
                return -1;  // Player 1 should play before player 2

            return 1;
        }

    }
}
